@@ -1,24 +1,24 @@
 /*****************************************************************************
  * smb.c: SMB input module
  *****************************************************************************
- * Copyright (C) 2001-2009 the VideoLAN team
- * $Id: c9b931477a697a5b694e9036c56f73ab34ad87a1 $
+ * Copyright (C) 2001-2009 VLC authors and VideoLAN
+ * $Id: 1e681aa276f765f33080e9ec8d893993a17d17f9 $
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -28,18 +28,10 @@
 # include "config.h"
 #endif
 
-#include <vlc_common.h>
-#include <vlc_fs.h>
-#include <vlc_plugin.h>
-#include <vlc_access.h>
-
-#ifdef WIN32
-#   ifdef HAVE_FCNTL_H
-#       include <fcntl.h>
-#   endif
-#   ifdef HAVE_SYS_STAT_H
-#       include <sys/stat.h>
-#   endif
+#include <errno.h>
+#ifdef _WIN32
+#   include <fcntl.h>
+#   include <sys/stat.h>
 #   include <io.h>
 #   define smbc_open(a,b,c) vlc_open(a,b,c)
 #   define smbc_fstat(a,b) _fstati64(a,b)
@@ -50,7 +42,10 @@
 #   include <libsmbclient.h>
 #endif
 
-#include <errno.h>
+#include <vlc_common.h>
+#include <vlc_fs.h>
+#include <vlc_plugin.h>
+#include <vlc_access.h>
 
 /*****************************************************************************
  * Module descriptor
@@ -98,7 +93,7 @@ struct access_sys_t
     int i_smb;
 };
 
-#ifdef WIN32
+#ifdef _WIN32
 static void Win32AddConnection( access_t *, char *, char *, char *, char * );
 #else
 static void smb_auth( const char *srv, const char *shr, char *wg, int wglen,
@@ -184,7 +179,7 @@ static int Open( vlc_object_t *p_this )
     if( !psz_domain ) psz_domain = var_InheritString( p_access, "smb-domain" );
     if( psz_domain && !*psz_domain ) { free( psz_domain ); psz_domain = NULL; }
 
-#ifdef WIN32
+#ifdef _WIN32
     if( psz_user )
         Win32AddConnection( p_access, psz_location, psz_user, psz_pwd, psz_domain);
     i_ret = asprintf( &psz_uri, "//%s", psz_location );
@@ -205,7 +200,7 @@ static int Open( vlc_object_t *p_this )
     if( i_ret == -1 )
         return VLC_ENOMEM;
 
-#ifndef WIN32
+#ifndef _WIN32
     if( smbc_init( smb_auth, 0 ) )
     {
         free( psz_uri );
@@ -348,7 +343,7 @@ static int Control( access_t *p_access, int i_query, va_list args )
     return VLC_SUCCESS;
 }
 
-#ifdef WIN32
+#ifdef _WIN32
 static void Win32AddConnection( access_t *p_access, char *psz_path,
                                 char *psz_user, char *psz_pwd,
                                 char *psz_domain )
@@ -392,4 +387,4 @@ static void Win32AddConnection( access_t *p_access, char *psz_path,
         msg_Dbg( p_access, "failed to connect to %s", psz_remote );
     }
 }
-#endif // WIN32
+#endif // _WIN32
