@@ -1,24 +1,24 @@
 /*****************************************************************************
  * trivial.c : trivial channel mixer plug-in (drops unwanted channels)
  *****************************************************************************
- * Copyright (C) 2002, 2006 the VideoLAN team
- * $Id: 9211751e50044156bf759c11de7bc6b02f6374ba $
+ * Copyright (C) 2002, 2006 VLC authors and VideoLAN
+ * $Id: 86b9de0632f86f87131743ada5ea3344d40ffb11 $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -46,7 +46,7 @@ static block_t *DoWork( filter_t *, block_t * );
  *****************************************************************************/
 vlc_module_begin ()
     set_description( N_("Audio filter for trivial channel mixing") )
-    set_capability( "audio filter", 1 )
+    set_capability( "audio converter", 1 )
     set_category( CAT_AUDIO )
     set_subcategory( SUBCAT_AUDIO_MISC )
     set_callbacks( Create, NULL )
@@ -65,8 +65,7 @@ static int Create( vlc_object_t *p_this )
                == p_filter->fmt_out.audio.i_original_channels)
           || p_filter->fmt_in.audio.i_format != p_filter->fmt_out.audio.i_format
           || p_filter->fmt_in.audio.i_rate != p_filter->fmt_out.audio.i_rate
-          || (p_filter->fmt_in.audio.i_format != VLC_CODEC_FL32
-               && p_filter->fmt_in.audio.i_format != VLC_CODEC_FI32) )
+          || p_filter->fmt_in.audio.i_format != VLC_CODEC_FL32 )
     {
         return VLC_EGENERIC;
     }
@@ -78,7 +77,7 @@ static int Create( vlc_object_t *p_this )
 /*****************************************************************************
  * SparseCopy: trivially downmix or upmix a buffer
  *****************************************************************************/
-static void SparseCopy( int32_t * p_dest, const int32_t * p_src, size_t i_len,
+static void SparseCopy( float * p_dest, const float * p_src, size_t i_len,
                         int i_output_stride, int i_input_stride )
 {
     int i;
@@ -110,7 +109,7 @@ static block_t *DoWork( filter_t * p_filter, block_t * p_in_buf )
     }
     else
     {
-        p_out_buf = filter_NewAudioBuffer( p_filter,
+        p_out_buf = block_Alloc(
                               p_in_buf->i_buffer / i_input_nb * i_output_nb );
         if( !p_out_buf )
             goto out;
@@ -120,8 +119,8 @@ static block_t *DoWork( filter_t * p_filter, block_t * p_in_buf )
         p_out_buf->i_length     = p_in_buf->i_length;
     }
 
-    int32_t * p_dest = (int32_t *)p_out_buf->p_buffer;
-    const int32_t * p_src = (int32_t *)p_in_buf->p_buffer;
+    float * p_dest = (float *)p_out_buf->p_buffer;
+    const float * p_src = (float *)p_in_buf->p_buffer;
 
     if ( (p_filter->fmt_out.audio.i_original_channels & AOUT_CHAN_PHYSMASK)
                 != (p_filter->fmt_in.audio.i_original_channels & AOUT_CHAN_PHYSMASK)
@@ -164,7 +163,7 @@ static block_t *DoWork( filter_t * p_filter, block_t * p_in_buf )
         int i;
         for ( i = p_in_buf->i_nb_samples; i--; )
         {
-            int32_t i_tmp = p_src[0];
+            float i_tmp = p_src[0];
             p_dest[0] = p_src[1];
             p_dest[1] = i_tmp;
 
