@@ -1,24 +1,24 @@
 /*****************************************************************************
  * bandlimited.c : band-limited interpolation resampler
  *****************************************************************************
- * Copyright (C) 2002, 2006 the VideoLAN team
- * $Id: 5368fc400954eb6473ef264efc6d5a90b1be1df9 $
+ * Copyright (C) 2002, 2006 VLC authors and VideoLAN
+ * $Id: f86be17da148af4ebc2039b3f0876a10d1c1f883 $
  *
  * Authors: Gildas Bazin <gbazin@netcourrier.com>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -87,7 +87,11 @@ vlc_module_begin ()
     set_category( CAT_AUDIO )
     set_subcategory( SUBCAT_AUDIO_MISC )
     set_description( N_("Audio filter for band-limited interpolation resampling") )
-    set_capability( "audio filter", 20 )
+    set_capability( "audio converter", 20 )
+    set_callbacks( OpenFilter, CloseFilter )
+
+    add_submodule()
+    set_capability( "audio resampler", 20 )
     set_callbacks( OpenFilter, CloseFilter )
 vlc_module_end ()
 
@@ -141,7 +145,7 @@ static block_t *Resample( filter_t * p_filter, block_t * p_in_buf )
     size_t i_out_size = i_bytes_per_frame * ( 1 + ( p_in_buf->i_nb_samples *
               p_filter->fmt_out.audio.i_rate / p_filter->fmt_in.audio.i_rate) )
             + p_filter->p_sys->i_buf_size;
-    block_t *p_out_buf = filter_NewAudioBuffer( p_filter, i_out_size );
+    block_t *p_out_buf = block_Alloc( i_out_size );
     if( !p_out_buf )
     {
         block_Release( p_in_buf );
@@ -296,13 +300,6 @@ static int OpenFilter( vlc_object_t *p_this )
         return VLC_EGENERIC;
     }
 
-#if !defined( SYS_DARWIN )
-    if( !var_InheritBool( p_this, "hq-resampling" ) )
-    {
-        return VLC_EGENERIC;
-    }
-#endif
-
     /* Allocate the memory needed to store the module's structure */
     p_filter->p_sys = p_sys = malloc( sizeof(struct filter_sys_t) );
     if( p_sys == NULL )
