@@ -2,7 +2,7 @@
  * video.c: video decoder using the libavcodec library
  *****************************************************************************
  * Copyright (C) 1999-2001 VLC authors and VideoLAN
- * $Id: 2fd48affcd61920340d43486a8d32292527ee2a2 $
+ * $Id: 0d8c42b117267d64de416b0314b6bdd3972c5d72 $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Gildas Bazin <gbazin@videolan.org>
@@ -217,6 +217,11 @@ int InitVideoDec( decoder_t *p_dec, AVCodecContext *p_context,
     if( var_CreateGetBool( p_dec, "grayscale" ) )
         p_sys->p_context->flags |= CODEC_FLAG_GRAY;
 
+    /* ***** Output always the frames ***** */
+#if LIBAVCODEC_VERSION_CHECK(55, 23, 1, 40, 101)
+    p_sys->p_context->flags |= CODEC_FLAG_OUTPUT_CORRUPT;
+#endif
+
     i_val = var_CreateGetInteger( p_dec, "avcodec-vismv" );
     if( i_val ) p_sys->p_context->debug_mv = i_val;
 
