@@ -3,8 +3,8 @@
  *            DVB subtitles encoder (developed for Anevia, www.anevia.com)
  *****************************************************************************
  * Copyright (C) 2003 ANEVIA
- * Copyright (C) 2003-2009 the VideoLAN team
- * $Id: 0d9b61b9fcda56825c2d986c315ce7a4d73e4e03 $
+ * Copyright (C) 2003-2009 VLC authors and VideoLAN
+ * $Id: f26013d79595f0b8e998f21217250328332350f3 $
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *          Damien LUCAS <damien.lucas@anevia.com>
@@ -13,19 +13,19 @@
  *          Derk-Jan Hartman <hartman #at# videolan dot org>
  *          Simon Hailes <simon _a_ screen.subtitling.com>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -817,7 +817,7 @@ static void decode_page_composition( decoder_t *p_dec, bs_t *s )
     if( p_sys->p_page->i_region_defs == 0 ) return;
 
     p_sys->p_page->p_region_defs =
-        malloc( p_sys->p_page->i_region_defs * sizeof(dvbsub_region_t) );
+        malloc( p_sys->p_page->i_region_defs * sizeof(dvbsub_regiondef_t) );
     if( p_sys->p_page->p_region_defs )
     {
         for( i = 0; i < p_sys->p_page->i_region_defs; i++ )
@@ -1475,7 +1475,7 @@ static subpicture_t *render( decoder_t *p_dec )
     decoder_sys_t *p_sys = p_dec->p_sys;
     subpicture_t *p_spu;
     subpicture_region_t **pp_spu_region;
-    int i, j, i_timeout = 0;
+    int i, j;
     int i_base_x;
     int i_base_y;
 
@@ -1536,8 +1536,6 @@ static subpicture_t *render( decoder_t *p_dec )
         video_palette_t palette;
         int i_pitch;
 
-        i_timeout = p_sys->p_page->i_timeout;
-
         p_regiondef = &p_sys->p_page->p_region_defs[i];
 
         /* Find associated region */
@@ -1996,7 +1994,7 @@ static block_t *Encode( encoder_t *p_enc, subpicture_t *p_subpic )
 #ifdef DEBUG_DVBSUB
     msg_Dbg( p_enc, "encoding subpicture" );
 #endif
-    p_block = block_New( p_enc, 64000 );
+    p_block = block_Alloc( 64000 );
     bs_init( s, p_block->p_buffer, p_block->i_buffer );
 
     bs_write( s, 8, 0x20 ); /* Data identifier */
@@ -2023,7 +2021,7 @@ static block_t *Encode( encoder_t *p_enc, subpicture_t *p_subpic )
         p_block->i_length = p_subpic->i_stop - p_subpic->i_start;
 
         /* Send another (empty) subtitle to signal the end of display */
-        p_block_stop = block_New( p_enc, 64000 );
+        p_block_stop = block_Alloc( 64000 );
         bs_init( s, p_block_stop->p_buffer, p_block_stop->i_buffer );
         bs_write( s, 8, 0x20 ); /* Data identifier */
         bs_write( s, 8, 0x0 );  /* Subtitle stream id */
