@@ -2,7 +2,7 @@
  * zvbi.c : VBI and Teletext PES demux and decoder using libzvbi
  *****************************************************************************
  * Copyright (C) 2007, M2X
- * $Id: 2a1e1310d055b2ee9cc24b7097ed22e85bb24e8c $
+ * $Id: fdf326f15991ba133e8ea213f13543c876965557 $
  *
  * Authors: Derk-Jan Hartman <djhartman at m2x dot nl>
  *          Jean-Paul Saman <jpsaman at m2x dot nl>
@@ -182,7 +182,7 @@ static subpicture_t *Subpicture( decoder_t *p_dec, video_format_t *p_fmt,
                                  int i_align, mtime_t i_pts );
 
 static void EventHandler( vbi_event *ev, void *user_data );
-static int OpaquePage( picture_t *p_src, const vbi_page *p_page,
+static int OpaquePage( picture_t *p_src, const vbi_page p_page,
                        const video_format_t fmt, bool b_opaque, const int text_offset );
 static int get_first_visible_row( vbi_char *p_text, int rows, int columns);
 static int get_last_visible_row( vbi_char *p_text, int rows, int columns);
@@ -462,7 +462,7 @@ static subpicture_t *Decode( decoder_t *p_dec, block_t **pp_block )
         memcpy( p_sys->nav_link, &p_page.nav_link, sizeof( p_sys->nav_link )) ;
         vlc_mutex_unlock( &p_sys->lock );
 
-        OpaquePage( p_pic, &p_page, fmt, b_opaque, i_first_row * p_page.columns );
+        OpaquePage( p_pic, p_page, fmt, b_opaque, i_first_row * p_page.columns );
     }
 
 exit:
@@ -610,7 +610,7 @@ static int get_last_visible_row( vbi_char *p_text, int rows, int columns)
     return 0;
 }
 
-static int OpaquePage( picture_t *p_src, const vbi_page *p_page,
+static int OpaquePage( picture_t *p_src, const vbi_page p_page,
                        const video_format_t fmt, bool b_opaque, const int text_offset )
 {
     unsigned int    x, y;
@@ -622,8 +622,8 @@ static int OpaquePage( picture_t *p_src, const vbi_page *p_page,
     {
         for( x = 0; x < fmt.i_width; x++ )
         {
-            const vbi_opacity opacity = p_page->text[ text_offset + y/10 * p_page->columns + x/12 ].opacity;
-            const int background = p_page->text[ text_offset + y/10 * p_page->columns + x/12 ].background;
+            const vbi_opacity opacity = p_page.text[ text_offset + y/10 * p_page.columns + x/12 ].opacity;
+            const int background = p_page.text[ text_offset + y/10 * p_page.columns + x/12 ].background;
             uint32_t *p_pixel = (uint32_t*)&p_src->p->p_pixels[y * p_src->p->i_pitch + 4*x];
 
             switch( opacity )
@@ -643,7 +643,7 @@ static int OpaquePage( picture_t *p_src, const vbi_page *p_page,
                     break;
             /* Full text transparency. only foreground color is show */
             case VBI_TRANSPARENT_FULL:
-                if( (*p_pixel) == (0xff000000 | p_page->color_map[background] ) )
+                if( (*p_pixel) == (0xff000000 | p_page.color_map[background] ) )
                     *p_pixel = 0;
                 break;
             }
