@@ -1,13 +1,15 @@
 /*****************************************************************************
- * dbus-root.c : dbus control module (mpris v1.0) - root object
+ * dbus_root.c : dbus control module (mpris v1.0) - root object
  *****************************************************************************
  * Copyright © 2006-2008 Rafaël Carré
  * Copyright © 2007-2011 Mirsal Ennaime
  * Copyright © 2009-2011 The VideoLAN team
- * $Id: a96128ee774c89c0f252a79ca0e333a45be1b934 $
+ * Copyright © 2013      Alex Merry
+ * $Id: 7702034164a791549b4745a58d56f293e9740716 $
  *
  * Authors:    Mirsal Ennaime <mirsal at mirsal fr>
  *             Rafaël Carré <funman at videolanorg>
+ *             Alex Merry <dev at randomguy3 me uk>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,6 +32,10 @@
 
 #include <vlc_common.h>
 #include <vlc_interface.h>
+#include <vlc_input.h>
+#include <vlc_vout.h>
+#include <vlc_plugin.h>
+#include <vlc_playlist.h>
 
 #include <unistd.h>
 #include <limits.h>
@@ -74,22 +80,74 @@ MarshalIdentity( intf_thread_t *p_intf, DBusMessageIter *container )
     VLC_UNUSED( p_intf );
     const char *psz_id = _("VLC media player");
 
-    dbus_message_iter_append_basic( container, DBUS_TYPE_STRING, &psz_id );
+    if (!dbus_message_iter_append_basic( container, DBUS_TYPE_STRING, &psz_id ))
+        return VLC_ENOMEM;
+
     return VLC_SUCCESS;
 }
 
-DBUS_METHOD( Identity )
+static int
+MarshalCanSetFullscreen( intf_thread_t *p_intf, DBusMessageIter *container )
+{
+    input_thread_t *p_input = NULL;
+    dbus_bool_t     b_ret   = FALSE;
+
+    if (p_intf->p_sys->p_input)
+    {
+        p_input = (input_thread_t*) vlc_object_hold( p_intf->p_sys->p_input );
+        vout_thread_t* p_vout = input_GetVout( p_input );
+        vlc_object_release( p_input );
+
+        if ( p_vout )
+        {
+            b_ret = TRUE;
+            vlc_object_release( p_vout );
+        }
+    }
+
+    if (!dbus_message_iter_append_basic( container, DBUS_TYPE_BOOLEAN, &b_ret ))
+        return VLC_ENOMEM;
+
+    return VLC_SUCCESS;
+}
+
+static int
+MarshalFullscreen( intf_thread_t *p_intf, DBusMessageIter *container )
+{
+    dbus_bool_t b_fullscreen;
+
+    if ( p_intf->p_sys->p_playlist )
+        b_fullscreen = var_GetBool( p_intf->p_sys->p_playlist , "fullscreen" );
+    else
+        b_fullscreen = FALSE;
+
+    if (!dbus_message_iter_append_basic( container,
+            DBUS_TYPE_BOOLEAN, &b_fullscreen ))
+        return VLC_ENOMEM;
+
+    return VLC_SUCCESS;
+}
+
+DBUS_METHOD( FullscreenSet )
 {
     REPLY_INIT;
-    OUT_ARGUMENTS;
+    dbus_bool_t b_fullscreen;
+    input_thread_t *p_input = NULL;
 
-    DBusMessageIter v;
-    dbus_message_iter_open_container( &args, DBUS_TYPE_VARIANT, "s", &v );
+    if( VLC_SUCCESS != DemarshalSetPropertyValue( p_from, &b_fullscreen ) )
+        return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 
-    MarshalIdentity( p_this, &v );
+    if (INTF->p_sys->p_input)
+    {
+        p_input = (input_thread_t*) vlc_object_hold( INTF->p_sys->p_input );
+        vout_thread_t* p_vout = input_GetVout( p_input );
+        vlc_object_release( p_input );
 
-    if( !dbus_message_iter_close_container( &args, &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
+        if ( p_vout )
+            var_SetBool( p_vout, "fullscreen", ( b_fullscreen == TRUE ) );
+        if ( PL )
+            var_SetBool( PL , "fullscreen", ( b_fullscreen == TRUE ) );
+    }
 
     REPLY_SEND;
 }
@@ -100,104 +158,48 @@ MarshalCanQuit( intf_thread_t *p_intf, DBusMessageIter *container )
     VLC_UNUSED( p_intf );
     const dbus_bool_t b_ret = TRUE;
 
-    dbus_message_iter_append_basic( container, DBUS_TYPE_BOOLEAN, &b_ret );
+    if (!dbus_message_iter_append_basic( container, DBUS_TYPE_BOOLEAN, &b_ret ))
+        return VLC_ENOMEM;
+
     return VLC_SUCCESS;
 }
 
-DBUS_METHOD( CanQuit )
-{
-    REPLY_INIT;
-    OUT_ARGUMENTS;
-
-    DBusMessageIter v;
-    dbus_message_iter_open_container( &args, DBUS_TYPE_VARIANT, "b", &v );
-
-    MarshalCanQuit( p_this, &v );
-
-    if( !dbus_message_iter_close_container( &args, &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    REPLY_SEND;
-}
-
 static int
 MarshalCanRaise( intf_thread_t *p_intf, DBusMessageIter *container )
 {
     VLC_UNUSED( p_intf );
     const dbus_bool_t b_ret = FALSE;
 
-    dbus_message_iter_append_basic( container, DBUS_TYPE_BOOLEAN, &b_ret );
+    if (!dbus_message_iter_append_basic( container, DBUS_TYPE_BOOLEAN, &b_ret ))
+        return VLC_ENOMEM;
+
     return VLC_SUCCESS;
 }
 
-DBUS_METHOD( CanRaise )
-{
-    REPLY_INIT;
-    OUT_ARGUMENTS;
-
-    DBusMessageIter v;
-    dbus_message_iter_open_container( &args, DBUS_TYPE_VARIANT, "b", &v );
-
-    MarshalCanRaise( p_this, &v );
-
-    if( !dbus_message_iter_close_container( &args, &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    REPLY_SEND;
-}
-
 static int
 MarshalHasTrackList( intf_thread_t *p_intf, DBusMessageIter *container )
 {
     VLC_UNUSED( p_intf );
     const dbus_bool_t b_ret = FALSE;
 
-    dbus_message_iter_append_basic( container, DBUS_TYPE_BOOLEAN, &b_ret );
+    if (!dbus_message_iter_append_basic( container, DBUS_TYPE_BOOLEAN, &b_ret ))
+        return VLC_ENOMEM;
+
     return VLC_SUCCESS;
 }
 
-DBUS_METHOD( HasTrackList )
-{
-    REPLY_INIT;
-    OUT_ARGUMENTS;
-
-    DBusMessageIter v;
-    dbus_message_iter_open_container( &args, DBUS_TYPE_VARIANT, "b", &v );
-
-    MarshalHasTrackList( p_this, &v );
-
-    if( !dbus_message_iter_close_container( &args, &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    REPLY_SEND;
-}
-
 static int
 MarshalDesktopEntry( intf_thread_t *p_intf, DBusMessageIter *container )
 {
     VLC_UNUSED( p_intf );
     const char* psz_ret = PACKAGE;
 
-    dbus_message_iter_append_basic( container, DBUS_TYPE_STRING, &psz_ret );
+    if (!dbus_message_iter_append_basic( container, DBUS_TYPE_STRING, &psz_ret ))
+        return VLC_ENOMEM;
+
     return VLC_SUCCESS;
 }
 
-DBUS_METHOD( DesktopEntry )
-{
-    REPLY_INIT;
-    OUT_ARGUMENTS;
-
-    DBusMessageIter v;
-    dbus_message_iter_open_container( &args, DBUS_TYPE_VARIANT, "s", &v );
-
-    MarshalDesktopEntry( p_this, &v );
-
-    if( !dbus_message_iter_close_container( &args, &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    REPLY_SEND;
-}
-
 static int
 MarshalSupportedMimeTypes( intf_thread_t *p_intf, DBusMessageIter *container )
 {
@@ -227,23 +229,6 @@ MarshalSupportedMimeTypes( intf_thread_t *p_intf, DBusMessageIter *container )
     return VLC_SUCCESS;
 }
 
-DBUS_METHOD( SupportedMimeTypes )
-{
-    REPLY_INIT;
-    OUT_ARGUMENTS;
-
-    DBusMessageIter v;
-    dbus_message_iter_open_container( &args, DBUS_TYPE_VARIANT, "as", &v );
-
-    if( VLC_SUCCESS != MarshalSupportedMimeTypes( p_this, &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    if( !dbus_message_iter_close_container( &args, &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    REPLY_SEND;
-}
-
 static int
 MarshalSupportedUriSchemes( intf_thread_t *p_intf, DBusMessageIter *container )
 {
@@ -273,24 +258,6 @@ MarshalSupportedUriSchemes( intf_thread_t *p_intf, DBusMessageIter *container )
     return VLC_SUCCESS;
 }
 
-DBUS_METHOD( SupportedUriSchemes )
-{
-    VLC_UNUSED( p_this );
-    REPLY_INIT;
-    OUT_ARGUMENTS;
-
-    DBusMessageIter v;
-    dbus_message_iter_open_container( &args, DBUS_TYPE_VARIANT, "as", &v );
-
-    if( VLC_SUCCESS != MarshalSupportedUriSchemes( p_this, &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    if( !dbus_message_iter_close_container( &args, &v ) )
-        return DBUS_HANDLER_RESULT_NEED_MEMORY;
-
-    REPLY_SEND;
-}
-
 DBUS_METHOD( Quit )
 { /* exits vlc */
     REPLY_INIT;
@@ -306,11 +273,22 @@ DBUS_METHOD( Raise )
 }
 
 #define PROPERTY_MAPPING_BEGIN if( 0 ) {}
-#define PROPERTY_FUNC( interface, property, function ) \
-    else if( !strcmp( psz_interface_name, interface ) && \
-             !strcmp( psz_property_name,  property ) ) \
-        return function( p_conn, p_from, p_this );
-#define PROPERTY_MAPPING_END return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+#define PROPERTY_GET_FUNC( prop, signature ) \
+    else if( !strcmp( psz_property_name,  #prop ) ) { \
+        if( !dbus_message_iter_open_container( &args, DBUS_TYPE_VARIANT, signature, &v ) ) \
+            return DBUS_HANDLER_RESULT_NEED_MEMORY; \
+        if( VLC_SUCCESS != Marshal##prop( p_this, &v ) ) { \
+            dbus_message_iter_abandon_container( &args, &v ); \
+            return DBUS_HANDLER_RESULT_NEED_MEMORY; \
+        } \
+        if( !dbus_message_iter_close_container( &args, &v ) ) \
+            return DBUS_HANDLER_RESULT_NEED_MEMORY; \
+    }
+#define PROPERTY_SET_FUNC( prop ) \
+    else if( !strcmp( psz_property_name,  #prop ) ) { \
+        return prop##Set( p_conn, p_from, p_this ); \
+    }
+#define PROPERTY_MAPPING_END else { return DBUS_HANDLER_RESULT_NOT_YET_HANDLED; }
 
 DBUS_METHOD( GetProperty )
 {
@@ -336,61 +314,66 @@ DBUS_METHOD( GetProperty )
     msg_Dbg( (vlc_object_t*) p_this, "Getting property %s",
                                      psz_property_name );
 
+    if( strcmp( psz_interface_name, DBUS_MPRIS_ROOT_INTERFACE ) ) {
+        return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+    }
+
+    REPLY_INIT;
+    OUT_ARGUMENTS;
+    DBusMessageIter v;
+
     PROPERTY_MAPPING_BEGIN
-    PROPERTY_FUNC( DBUS_MPRIS_ROOT_INTERFACE, "Identity",            Identity )
-    PROPERTY_FUNC( DBUS_MPRIS_ROOT_INTERFACE, "DesktopEntry",        DesktopEntry )
-    PROPERTY_FUNC( DBUS_MPRIS_ROOT_INTERFACE, "SupportedMimeTypes",  SupportedMimeTypes )
-    PROPERTY_FUNC( DBUS_MPRIS_ROOT_INTERFACE, "SupportedUriSchemes", SupportedUriSchemes )
-    PROPERTY_FUNC( DBUS_MPRIS_ROOT_INTERFACE, "HasTrackList",        HasTrackList )
-    PROPERTY_FUNC( DBUS_MPRIS_ROOT_INTERFACE, "CanQuit",             CanQuit )
-    PROPERTY_FUNC( DBUS_MPRIS_ROOT_INTERFACE, "CanRaise",            CanRaise )
+    PROPERTY_GET_FUNC( Identity,            "s" )
+    PROPERTY_GET_FUNC( CanSetFullscreen,    "b" )
+    PROPERTY_GET_FUNC( Fullscreen,          "b" )
+    PROPERTY_GET_FUNC( CanQuit,             "b" )
+    PROPERTY_GET_FUNC( CanRaise,            "b" )
+    PROPERTY_GET_FUNC( HasTrackList,        "b" )
+    PROPERTY_GET_FUNC( DesktopEntry,        "s" )
+    PROPERTY_GET_FUNC( SupportedMimeTypes,  "as" )
+    PROPERTY_GET_FUNC( SupportedUriSchemes, "as" )
+    PROPERTY_MAPPING_END
+
+    REPLY_SEND;
+}
+
+DBUS_METHOD( SetProperty )
+{
+    DBusError error;
+
+    char *psz_interface_name = NULL;
+    char *psz_property_name  = NULL;
+
+    dbus_error_init( &error );
+    dbus_message_get_args( p_from, &error,
+            DBUS_TYPE_STRING, &psz_interface_name,
+            DBUS_TYPE_STRING, &psz_property_name,
+            DBUS_TYPE_INVALID );
+
+    if( dbus_error_is_set( &error ) )
+    {
+        msg_Err( (vlc_object_t*) p_this, "D-Bus message reading : %s",
+                                        error.message );
+        dbus_error_free( &error );
+        return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+    }
+
+    PROPERTY_MAPPING_BEGIN
+    PROPERTY_SET_FUNC( Fullscreen )
     PROPERTY_MAPPING_END
 }
 
 #undef PROPERTY_MAPPING_BEGIN
 #undef PROPERTY_GET_FUNC
+#undef PROPERTY_SET_FUNC
 #undef PROPERTY_MAPPING_END
 
-static int
-AddProperty( intf_thread_t *p_intf,
-             DBusMessageIter *p_container,
-             const char* psz_property_name,
-             const char* psz_signature,
-             int (*pf_marshaller) (intf_thread_t*, DBusMessageIter*) )
-{
-    DBusMessageIter entry, v;
-
-    if( !dbus_message_iter_open_container( p_container,
-                                           DBUS_TYPE_DICT_ENTRY, NULL,
-                                           &entry ) )
-        return VLC_ENOMEM;
-
-    if( !dbus_message_iter_append_basic( &entry,
-                                         DBUS_TYPE_STRING,
-                                         &psz_property_name ) )
-        return VLC_ENOMEM;
-
-    if( !dbus_message_iter_open_container( &entry,
-                                           DBUS_TYPE_VARIANT, psz_signature,
-                                           &v ) )
-        return VLC_ENOMEM;
-
-    if( VLC_SUCCESS != pf_marshaller( p_intf, &v ) )
-        return VLC_ENOMEM;
-
-    if( !dbus_message_iter_close_container( &entry, &v) )
-        return VLC_ENOMEM;
-
-    if( !dbus_message_iter_close_container( p_container, &entry ) )
-        return VLC_ENOMEM;
-
-    return VLC_SUCCESS;
-}
-
 #define ADD_PROPERTY( prop, signature ) \
     if( VLC_SUCCESS != AddProperty( (intf_thread_t*) p_this, \
-                &dict, #prop, signature, Marshal##prop ) ) \
-        return VLC_ENOMEM;
+                &dict, #prop, signature, Marshal##prop ) ) { \
+        dbus_message_iter_abandon_container( &args, &dict ); \
+        return VLC_ENOMEM; \
+    }
 
 DBUS_METHOD( GetAllProperties )
 {
@@ -417,7 +400,8 @@ DBUS_METHOD( GetAllProperties )
 
     msg_Dbg( (vlc_object_t*) p_this, "Getting All properties" );
 
-    dbus_message_iter_open_container( &args, DBUS_TYPE_ARRAY, "{sv}", &dict );
+    if( !dbus_message_iter_open_container( &args, DBUS_TYPE_ARRAY, "{sv}", &dict ) )
+        return DBUS_HANDLER_RESULT_NEED_MEMORY;
 
     ADD_PROPERTY( Identity,            "s"  );
     ADD_PROPERTY( DesktopEntry,        "s"  );
@@ -425,9 +409,13 @@ DBUS_METHOD( GetAllProperties )
     ADD_PROPERTY( SupportedUriSchemes, "as" );
     ADD_PROPERTY( HasTrackList,        "b"  );
     ADD_PROPERTY( CanQuit,             "b"  );
+    ADD_PROPERTY( CanSetFullscreen,    "b"  );
+    ADD_PROPERTY( Fullscreen,          "b"  );
     ADD_PROPERTY( CanRaise,            "b"  );
 
-    dbus_message_iter_close_container( &args, &dict );
+    if( !dbus_message_iter_close_container( &args, &dict ))
+        return DBUS_HANDLER_RESULT_NEED_MEMORY;
+
     REPLY_SEND;
 }
 
@@ -444,6 +432,7 @@ handle_root ( DBusConnection *p_conn, DBusMessage *p_from, void *p_this )
 {
     METHOD_MAPPING_BEGIN
     METHOD_FUNC( DBUS_INTERFACE_PROPERTIES, "Get",          GetProperty );
+    METHOD_FUNC( DBUS_INTERFACE_PROPERTIES, "Set",          SetProperty );
     METHOD_FUNC( DBUS_INTERFACE_PROPERTIES, "GetAll",       GetAllProperties );
     METHOD_FUNC( DBUS_MPRIS_ROOT_INTERFACE, "Quit",         Quit );
     METHOD_FUNC( DBUS_MPRIS_ROOT_INTERFACE, "Raise",        Raise );
@@ -453,3 +442,84 @@ handle_root ( DBusConnection *p_conn, DBusMessage *p_from, void *p_this )
 #undef METHOD_MAPPING_BEGIN
 #undef METHOD_FUNC
 #undef METHOD_MAPPING_END
+/**
+ * PropertiesChangedSignal() synthetizes and sends the
+ * org.freedesktop.DBus.Properties.PropertiesChanged signal
+ */
+
+#define PROPERTY_MAPPING_BEGIN if( 0 ) {}
+#define PROPERTY_ENTRY( prop, signature ) \
+    else if( !strcmp( ppsz_properties[i], #prop ) ) \
+    { \
+        if( VLC_SUCCESS != AddProperty( (intf_thread_t*) p_intf, \
+                    &changed_properties, #prop, signature, Marshal##prop ) ) \
+            return DBUS_HANDLER_RESULT_NEED_MEMORY; \
+    }
+#define PROPERTY_MAPPING_END else { return DBUS_HANDLER_RESULT_NOT_YET_HANDLED; }
+
+static DBusHandlerResult
+PropertiesChangedSignal( intf_thread_t    *p_intf,
+                         vlc_dictionary_t *p_changed_properties )
+{
+    DBusConnection  *p_conn = p_intf->p_sys->p_conn;
+    DBusMessageIter changed_properties, invalidated_properties;
+    const char *psz_interface_name = DBUS_MPRIS_ROOT_INTERFACE;
+    char **ppsz_properties = NULL;
+    int i_properties = 0;
+
+    SIGNAL_INIT( DBUS_INTERFACE_PROPERTIES,
+                 DBUS_MPRIS_OBJECT_PATH,
+                 "PropertiesChanged" );
+
+    OUT_ARGUMENTS;
+    ADD_STRING( &psz_interface_name );
+
+    if( !dbus_message_iter_open_container( &args, DBUS_TYPE_ARRAY, "{sv}",
+                                           &changed_properties ) )
+        return DBUS_HANDLER_RESULT_NEED_MEMORY;
+
+    i_properties = vlc_dictionary_keys_count( p_changed_properties );
+    ppsz_properties = vlc_dictionary_all_keys( p_changed_properties );
+
+    if( unlikely(!ppsz_properties) )
+    {
+        dbus_message_iter_abandon_container( &args, &changed_properties );
+        return DBUS_HANDLER_RESULT_NEED_MEMORY;
+    }
+
+    for( int i = 0; i < i_properties; i++ )
+    {
+        PROPERTY_MAPPING_BEGIN
+        PROPERTY_ENTRY( Fullscreen, "b" )
+        PROPERTY_MAPPING_END
+
+        free( ppsz_properties[i] );
+    }
+
+    if( !dbus_message_iter_close_container( &args, &changed_properties ) )
+        return DBUS_HANDLER_RESULT_NEED_MEMORY;
+
+    if( !dbus_message_iter_open_container( &args, DBUS_TYPE_ARRAY, "s",
+                                           &invalidated_properties ) )
+        return DBUS_HANDLER_RESULT_NEED_MEMORY;
+
+    if( !dbus_message_iter_close_container( &args, &invalidated_properties ) )
+        return DBUS_HANDLER_RESULT_NEED_MEMORY;
+
+    free( ppsz_properties );
+
+    SIGNAL_SEND;
+}
+
+/*****************************************************************************
+ * RootPropertiesChangedEmit: Emits the Seeked signal
+ *****************************************************************************/
+int RootPropertiesChangedEmit( intf_thread_t    *p_intf,
+                               vlc_dictionary_t *p_changed_properties )
+{
+    if( p_intf->p_sys->b_dead )
+        return VLC_SUCCESS;
+
+    PropertiesChangedSignal( p_intf, p_changed_properties );
+    return VLC_SUCCESS;
+}
