@@ -7,19 +7,19 @@
  * adapted from: http://lives.svn.sourceforge.net/viewvc/lives/trunk/lives-plugins
  * /plugins/decoders/ogg_theora_decoder.c
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -58,22 +58,6 @@ void oggseek_index_entries_free ( demux_index_entry_t *idx )
 }
 
 
-/* unlink and free idx. If idx is head of list, return new head */
-
-static demux_index_entry_t *index_entry_delete( demux_index_entry_t *idx )
-{
-    demux_index_entry_t *xidx = idx;
-
-    if ( idx->p_prev != NULL ) idx->p_prev->p_next = idx->p_next;
-    else xidx = idx->p_next;
-
-    if ( idx->p_next != NULL ) idx->p_next->p_prev = idx->p_prev;
-    free( idx );
-
-    return xidx;
-}
-
-
 /* internal function to create a new list member */
 
 static demux_index_entry_t *index_entry_new( void )
@@ -391,13 +375,11 @@ static int64_t find_last_frame (demux_t *p_demux, logical_stream_t *p_stream)
     int64_t i_kframe = 0;
     int64_t i_pos1;
     int64_t i_pos2;
-    int64_t i_serialno;
 
     demux_sys_t *p_sys  = p_demux->p_sys;
 
     i_pos1 = p_stream->i_data_start;
     i_pos2 = p_sys->i_total_length;
-    i_serialno = p_stream->os.serialno;
 
     i_start_pos = i_pos2 - OGGSEEK_BYTES_TO_READ;
 
@@ -875,8 +857,8 @@ int64_t oggseek_read_page( demux_t *p_demux )
     demux_sys_t *p_ogg = p_demux->p_sys  ;
     uint8_t header[PAGE_HEADER_BYTES+255];
     int i_nsegs;
-    int i_in_pos;
     int i;
+    int64_t i_in_pos;
     int64_t i_result;
     int i_page_size;
     char *buf;
@@ -930,8 +912,8 @@ int64_t oggseek_read_page( demux_t *p_demux )
 
     if ( ogg_sync_pageout( &p_ogg->oy, &p_ogg->current_page ) != 1 )
     {
-        msg_Err( p_demux , "Got invalid packet, read %"PRId64" of %i: %s",i_result,i_page_size,
-                 buf );
+        msg_Err( p_demux , "Got invalid packet, read %"PRId64" of %i: %s %"PRId64,
+                 i_result, i_page_size, buf, i_in_pos );
         return 0;
     }
 
