@@ -2,7 +2,7 @@
  * MainMenu.m: MacOS X interface module
  *****************************************************************************
  * Copyright (C) 2011-2013 Felix Paul Kühne
- * $Id: 17e748152bc992343f639cc638d2bb5107db61cc $
+ * $Id: 4c7a97783b45a9665bd8376632b27d28f2bd1d74 $
  *
  * Authors: Felix Paul Kühne <fkuehne -at- videolan -dot- org>
  *
@@ -743,12 +743,18 @@ static VLCMainMenu *_o_sharedInstance = nil;
     [[[VLCMain sharedInstance] playlist] setColumn: o_column state: i_new_state translationDict: o_ptc_translation_dict];
 }
 
-- (void)setPlaylistColumnTableState:(NSInteger)i_state forColumn:(NSString *)o_column
+- (BOOL)setPlaylistColumnTableState:(NSInteger)i_state forColumn:(NSString *)o_column
 {
-    NSInteger i_tag = [o_ptc_menuorder indexOfObject: o_column];
+    NSUInteger i_tag = [o_ptc_menuorder indexOfObject: o_column];
+    // prevent setting unknown columns
+    if(i_tag == NSNotFound)
+        return NO;
+
     [[o_mu_playlistTableColumns            itemWithTag: i_tag] setState: i_state];
     [[o_mu_playlistTableColumnsContextMenu itemWithTag: i_tag] setState: i_state];
     [[[VLCMain sharedInstance] playlist] setColumn: o_column state: i_state translationDict: o_ptc_translation_dict];
+
+    return YES;
 }
 
 #pragma mark -
@@ -884,7 +890,9 @@ static VLCMainMenu *_o_sharedInstance = nil;
     if (p_input) {
         vout_thread_t *p_vout = getVoutForActiveWindow();
         if (p_vout) {
-            var_ToggleBool(p_vout, "video-on-top");
+            BOOL b_fs = var_ToggleBool(p_vout, "video-on-top");
+            var_SetBool(pl_Get(p_intf), "video-on-top", b_fs);
+
             vlc_object_release(p_vout);
         }
         vlc_object_release(p_input);
@@ -1425,10 +1433,6 @@ static VLCMainMenu *_o_sharedInstance = nil;
     assert([data isKindOfClass:[VLCAutoGeneratedMenuContent class]]);
     VLCAutoGeneratedMenuContent *menuContent = (VLCAutoGeneratedMenuContent *)data;
 
-    /* Preserve settings across vouts via the playlist object: */
-    if (!strcmp([menuContent name], "fullscreen") || !strcmp([menuContent name], "video-on-top"))
-        var_Set(pl_Get(VLCIntf), [menuContent name] , [menuContent value]);
-
     p_object = [menuContent vlcObject];
 
     if (p_object != NULL) {
