@@ -1,8 +1,8 @@
 /*****************************************************************************
  * open.h: Open dialogues for VLC's MacOS X port
  *****************************************************************************
- * Copyright (C) 2002-2011 VLC authors and VideoLAN
- * $Id: 73c8d1b5a72f44dcd1ecfa572bba17aab09918fa $
+ * Copyright (C) 2002-2012 VLC authors and VideoLAN
+ * $Id: a932c878a702219c7e7fee8b1896c136154847da $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
  *          Christophe Massiot <massiot@via.ecp.fr>
@@ -24,14 +24,14 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
-#define kVLCMediaAudioCD "AudioCD"
-#define kVLCMediaDVD "DVD"
-#define kVLCMediaVCD "VCD"
-#define kVLCMediaSVCD "SVCD"
-#define kVLCMediaBD "Bluray"
-#define kVLCMediaVideoTSFolder "VIDEO_TS"
-#define kVLCMediaBDMVFolder "BDMV"
-#define kVLCMediaUnknown "Unknown"
+#define kVLCMediaAudioCD @"AudioCD"
+#define kVLCMediaDVD @"DVD"
+#define kVLCMediaVCD @"VCD"
+#define kVLCMediaSVCD @"SVCD"
+#define kVLCMediaBD @"Blu-ray"
+#define kVLCMediaVideoTSFolder @"VIDEO_TS"
+#define kVLCMediaBDMVFolder @"BDMV"
+#define kVLCMediaUnknown @"Unknown"
 
 /*****************************************************************************
  * Intf_Open interface
@@ -65,6 +65,11 @@
     IBOutlet id o_file_slave_icon_well;
     IBOutlet id o_file_subtitles_filename_lbl;
     IBOutlet id o_file_subtitles_icon_well;
+    IBOutlet id o_file_custom_timing_ckb;
+    IBOutlet id o_file_starttime_fld;
+    IBOutlet id o_file_starttime_lbl;
+    IBOutlet id o_file_stoptime_fld;
+    IBOutlet id o_file_stoptime_lbl;
 
     /* open disc */
     IBOutlet id o_disc_selector_pop;
@@ -190,6 +195,8 @@
     IBOutlet id o_screen_fps_fld;
     IBOutlet id o_screen_fps_lbl;
     IBOutlet id o_screen_fps_stp;
+    IBOutlet id o_screen_screen_lbl;
+    IBOutlet id o_screen_screen_pop;
     IBOutlet id o_screen_left_fld;
     IBOutlet id o_screen_left_lbl;
     IBOutlet id o_screen_left_stp;
@@ -203,12 +210,15 @@
     IBOutlet id o_screen_height_lbl;
     IBOutlet id o_screen_height_stp;
     IBOutlet id o_screen_follow_mouse_ckb;
+    IBOutlet id o_screen_qtk_audio_pop;
+    IBOutlet id o_screen_qtk_audio_ckb;
 
     /* QTK support */
     IBOutlet id o_qtk_view;
-    IBOutlet id o_qtk_long_lbl;
-    IBOutlet id o_qtk_device_pop;
-    IBOutlet id o_qtk_label_view;
+    IBOutlet id o_qtk_video_device_pop;
+    IBOutlet id o_qtk_video_ckb;
+    IBOutlet id o_qtk_audio_device_pop;
+    IBOutlet id o_qtk_audio_ckb;
     IBOutlet id o_capture_width_lbl;
     IBOutlet id o_capture_width_fld;
     IBOutlet id o_capture_width_stp;
@@ -217,29 +227,38 @@
     IBOutlet id o_capture_height_stp;
 
     NSArray         *qtkvideoDevices;
+    NSArray         *qtkaudioDevices;
     NSString        *qtk_currdevice_uid;
+    NSString        *qtkaudio_currdevice_uid;
 
     BOOL b_autoplay;
     BOOL b_nodvdmenus;
     id o_currentOpticalMediaView;
     id o_currentOpticalMediaIconView;
-    NSMutableArray *o_opticalDevices;
+    NSMutableArray *o_allMediaDevices;
+    NSArray *o_opticalDevices;
     NSMutableArray *o_specialMediaFolders;
     NSString *o_file_path;
     id o_currentCaptureView;
     NSString *o_file_slave_path;
     NSString *o_sub_path;
     NSString *o_mrl;
-    intf_thread_t * p_intf;
+    NSMutableArray *o_displayInfos;
 }
 
 + (VLCOpen *)sharedInstance;
 
-- (void)setMRL:(NSString *)mrl;
-- (NSString *)MRL;
+@property (readwrite, assign) NSString *MRL;
+@property (readonly) NSArray *qtkvideoDevices;
+@property (readonly) NSArray *qtkaudioDevices;
 
-- (NSArray *)qtkvideoDevices;
-- (void)qtkrefreshDevices;
+/* text field / stepper binding values - subs panel */
+@property (nonatomic) float fileSubDelay;
+@property (nonatomic) float fileSubFps;
+
+
+- (void)qtkrefreshVideoDevices;
+- (void)qtkrefreshAudioDevices;
 
 - (void)setSubPanel;
 - (void)openTarget:(int)i_type;
@@ -247,11 +266,11 @@
 - (void)textFieldWasClicked:(NSNotification *)o_notification;
 - (IBAction)expandMRLfieldAction:(id)sender;
 - (IBAction)inputSlaveAction:(id)sender;
+- (IBAction)fileTimeCustomization:(id)sender;
 
 - (void)openFileGeneric;
 - (void)openFilePathChanged:(NSNotification *)o_notification;
 - (IBAction)openFileBrowse:(id)sender;
-- (void)pathChosenInPanel: (NSOpenPanel *)sheet withReturn:(int)returnCode contextInfo:(void *)contextInfo;
 - (IBAction)openFileStreamChanged:(id)sender;
 
 - (void)openDisc;
@@ -260,7 +279,7 @@
 - (IBAction)openSpecialMediaFolder:(id)sender;
 - (IBAction)dvdreadOptionChanged:(id)sender;
 - (IBAction)vcdOptionChanged:(id)sender;
-- (char *)getVolumeTypeFromMountPath:(NSString *)mountPath;
+- (NSString *)getVolumeTypeFromMountPath:(NSString *)mountPath;
 - (NSString *)getBSDNodeFromMountPath:(NSString *)mountPath;
 
 - (void)openNet;
@@ -273,20 +292,21 @@
 - (void)showCaptureView: theView;
 - (IBAction)openCaptureModeChanged:(id)sender;
 - (IBAction)qtkChanged:(id)sender;
+- (IBAction)qtkAudioChanged:(id)sender;
+- (IBAction)qtkToggleUIElements:(id)sender;
+- (IBAction)screenChanged:(id)sender;
 - (IBAction)eyetvSwitchChannel:(id)sender;
 - (IBAction)eyetvLaunch:(id)sender;
 - (IBAction)eyetvGetPlugin:(id)sender;
 - (void)eyetvChanged:(NSNotification *)o_notification;
 - (void)setupChannelInfo;
-- (IBAction)screenStepperChanged:(id)sender;
 - (void)screenFPSfieldChanged:(NSNotification *)o_notification;
 
 - (IBAction)subsChanged:(id)sender;
 - (IBAction)subSettings:(id)sender;
 - (IBAction)subFileBrowse:(id)sender;
 - (IBAction)subOverride:(id)sender;
-- (IBAction)subDelayStepperChanged:(id)sender;
-- (IBAction)subFpsStepperChanged:(id)sender;
+
 - (IBAction)subCloseSheet:(id)sender;
 
 - (IBAction)panelCancel:(id)sender;
@@ -296,7 +316,5 @@
 @end
 
 @interface VLCOpenTextField : NSTextField
-{
-}
 - (void)mouseDown:(NSEvent *)theEvent;
 @end
