@@ -2,7 +2,7 @@
  * playlist_model.cpp : Manage playlist model
  ****************************************************************************
  * Copyright (C) 2006-2011 the VideoLAN team
- * $Id: b020b638e612020bdbb174ba054ef634c7ce6e7b $
+ * $Id: 932a4b07788e9876e3cfdaaeb76992b923063963 $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *          Ilkka Ollakkka <ileoo (at) videolan dot org>
@@ -29,10 +29,7 @@
 
 #include "qt4.hpp"
 #include "components/playlist/playlist_model.hpp"
-#include "dialogs_provider.hpp"                         /* THEDP */
 #include "input_manager.hpp"                            /* THEMIM */
-#include "dialogs/mediainfo.hpp"                        /* MediaInfo Dialog */
-#include "dialogs/playlist.hpp"                         /* Playlist Dialog */
 
 #include <vlc_intf_strings.h>                           /* I_DIR */
 
@@ -42,18 +39,9 @@
 #include <assert.h>
 #include <QIcon>
 #include <QFont>
-#include <QMenu>
-#include <QUrl>
-#include <QFileInfo>
-#include <QDesktopServices>
-#include <QInputDialog>
-#include <QSignalMapper>
-
-#define I_NEW_DIR \
-    I_DIR_OR_FOLDER( N_("Create Directory"), N_( "Create Folder" ) )
-#define I_NEW_DIR_NAME \
-    I_DIR_OR_FOLDER( N_( "Enter name for new directory:" ), \
-                     N_( "Enter name for new folder:" ) )
+#include <QTimer>
+#include <QAction>
+#include <QBuffer>
 
 QIcon PLModel::icons[ITEM_TYPE_NUMBER];
 
@@ -70,15 +58,13 @@ PLModel::PLModel( playlist_t *_p_playlist,  /* THEPL */
     p_playlist        = _p_playlist;
     i_cached_id       = -1;
     i_cached_input_id = -1;
-    i_popup_item      = i_popup_parent = -1;
-    sortingMenu       = NULL;
 
     rootItem          = NULL; /* PLItem rootItem, will be set in rebuild( ) */
     latestSearch      = QString();
 
     /* Icons initialization */
 #define ADD_ICON(type, x) icons[ITEM_TYPE_##type] = QIcon( x )
-    ADD_ICON( UNKNOWN , type_unknown_xpm );
+    ADD_ICON( UNKNOWN , QPixmap( type_unknown_xpm ) );
     ADD_ICON( FILE, ":/type/file" );
     ADD_ICON( DIRECTORY, ":/type/directory" );
     ADD_ICON( DISC, ":/type/disc" );
@@ -89,8 +75,6 @@ PLModel::PLModel( playlist_t *_p_playlist,  /* THEPL */
     ADD_ICON( NODE, ":/type/node" );
 #undef ADD_ICON
 
-    i_zoom = getSettings()->value( "Playlist/zoom", 0 ).toInt();
-
     rebuild( p_root );
     DCONNECT( THEMIM->getIM(), metaChanged( input_item_t *),
               this, processInputItemUpdate( input_item_t *) );
@@ -100,13 +84,12 @@ PLModel::PLModel( playlist_t *_p_playlist,  /* THEPL */
              this, processItemAppend( int, int ) );
     CONNECT( THEMIM, playlistItemRemoved( int ),
              this, processItemRemoval( int ) );
+    CONNECT( &insertBufferCommitTimer, timeout(), this, commitBufferedRowInserts() );
 }
 
 PLModel::~PLModel()
 {
-    getSettings()->setValue( "Playlist/zoom", i_zoom );
     delete rootItem;
-    delete sortingMenu;
 }
 
 Qt::DropActions PLModel::supportedDropActions() const
@@ -149,7 +132,7 @@ bool modelIndexLessThen( const QModelIndex &i1, const QModelIndex &i2 )
     if( !i1.isValid() || !i2.isValid() ) return false;
     PLItem *item1 = static_cast<PLItem*>( i1.internalPointer() );
     PLItem *item2 = static_cast<PLItem*>( i2.internalPointer() );
-    if( item1->parent() == item2->parent() ) return i1.row() < i2.row();
+    if( item1->hasSameParent( item2 ) ) return i1.row() < i2.row();
     else return *item1 < *item2;
 }
 
@@ -165,11 +148,11 @@ QMimeData *PLModel::mimeData( const QModelIndexList &indexes ) const
 
     qSort(list.begin(), list.end(), modelIndexLessThen);
 
-    PLItem *item = NULL;
+    AbstractPLItem *item = NULL;
     foreach( const QModelIndex &index, list ) {
         if( item )
         {
-            PLItem *testee = getItem( index );
+            AbstractPLItem *testee = getItem( index );
             while( testee->parent() )
             {
                 if( testee->parent() == item ||
@@ -182,7 +165,7 @@ QMimeData *PLModel::mimeData( const QModelIndexList &indexes ) const
         else
             item = getItem( index );
 
-        plMimeData->appendItem( item->inputItem() );
+        plMimeData->appendItem( static_cast<PLItem*>(item)->inputItem() );
     }
 
     return plMimeData;
@@ -233,7 +216,10 @@ void PLModel::dropMove( const PlMimeData * plMimeData, PLItem *target, int row )
 {
     QList<input_item_t*> inputItems = plMimeData->inputItems();
     QList<PLItem*> model_items;
-    playlist_item_t *pp_items[inputItems.count()];
+    playlist_item_t **pp_items;
+    pp_items = (playlist_item_t **)
+               calloc( inputItems.count(), sizeof( playlist_item_t* ) );
+    if ( !pp_items ) return;
 
     PL_LOCK;
 
@@ -242,7 +228,9 @@ void PLModel::dropMove( const PlMimeData * plMimeData, PLItem *target, int row )
 
     if( !p_parent || row > p_parent->i_children )
     {
-        PL_UNLOCK; return;
+        PL_UNLOCK;
+        free( pp_items );
+        return;
     }
 
     int new_pos = row == -1 ? p_parent->i_children : row;
@@ -260,12 +248,14 @@ void PLModel::dropMove( const PlMimeData * plMimeData, PLItem *target, int row )
         /* Better not try to move a node into itself.
            Abort the whole operation in that case,
            because it is ambiguous. */
-        PLItem *climber = target;
+        AbstractPLItem *climber = target;
         while( climber )
         {
             if( climber == item )
             {
-                PL_UNLOCK; return;
+                PL_UNLOCK;
+                free( pp_items );
+                return;
             }
             climber = climber->parent();
         }
@@ -281,7 +271,9 @@ void PLModel::dropMove( const PlMimeData * plMimeData, PLItem *target, int row )
 
     if( model_items.isEmpty() )
     {
-        PL_UNLOCK; return;
+        PL_UNLOCK;
+        free( pp_items );
+        return;
     }
 
     playlist_TreeMoveMany( p_playlist, i, pp_items, p_parent, new_pos );
@@ -292,6 +284,7 @@ void PLModel::dropMove( const PlMimeData * plMimeData, PLItem *target, int row )
         takeItem( item );
 
     insertChildren( target, model_items, model_pos );
+    free( pp_items );
 }
 
 /* remove item with its id */
@@ -332,7 +325,7 @@ void PLModel::activateItem( playlist_item_t *p_item )
 QVariant PLModel::data( const QModelIndex &index, const int role ) const
 {
     if( !index.isValid() ) return QVariant();
-    const PLItem *item = getItem( index );
+    PLItem *item = getItem( index );
     if( role == Qt::DisplayRole )
     {
         int metadata = columnToMeta( index.column() );
@@ -371,11 +364,43 @@ QVariant PLModel::data( const QModelIndex &index, const int role ) const
     }
     else if( role == Qt::FontRole )
     {
-        QFont f;
-        f.setPointSize( __MAX( f.pointSize() + i_zoom, 4 ) );
-        if( isCurrent( index ) )
-            f.setBold( true );
-        return QVariant( f );
+        return QVariant( QFont() );
+    }
+    else if( role == Qt::ToolTipRole )
+    {
+        int i_art_policy = var_GetInteger( p_playlist, "album-art" );
+        QString artUrl;
+        /* FIXME: Skip, as we don't want the pixmap and do not know the cached art file */
+        if ( i_art_policy == ALBUM_ART_ALL )
+            artUrl = getArtUrl( index );
+        if ( artUrl.isEmpty() ) artUrl = ":/noart";
+        QString duration = qtr( "unknown" );
+        QString name;
+        PL_LOCK;
+        input_item_t *p_item = item->inputItem();
+        if ( !p_item )
+        {
+            PL_UNLOCK;
+            return QVariant();
+        }
+        if ( p_item->i_duration > 0 )
+        {
+            char *psz = psz_column_meta( item->inputItem(), COLUMN_DURATION );
+            duration = qfu( psz );
+            free( psz );
+        }
+        name = qfu( p_item->psz_name );
+        PL_UNLOCK;
+        QPixmap image = getArtPixmap( index, QSize( 128, 128 ) );
+        QByteArray bytes;
+        QBuffer buffer( &bytes );
+        buffer.open( QIODevice::WriteOnly );
+        image.save(&buffer, "BMP"); /* uncompressed, see qpixmap#reading-and-writing-image-files */
+        return QVariant( QString("<img width=\"128\" height=\"128\" align=\"left\" src=\"data:image/bmp;base64,%1\"/><div><b>%2</b><br/>%3</div>")
+                         .arg( bytes.toBase64().constData() )
+                         .arg( name )
+                         .arg( qtr("Duration") + ": " + duration )
+                        );
     }
     else if( role == Qt::BackgroundRole && isCurrent( index ) )
     {
@@ -430,6 +455,51 @@ int PLModel::itemId( const QModelIndex &index ) const
     return getItem( index )->id();
 }
 
+input_item_t * PLModel::getInputItem( const QModelIndex &index ) const
+{
+    return getItem( index )->inputItem();
+}
+
+QString PLModel::getURI( const QModelIndex &index ) const
+{
+    QString uri;
+    input_item_t *p_item = getItem( index )->inputItem();
+    /* no PL lock as item gets refcount +1 from PLItem, which only depends of events */
+    vlc_mutex_lock( &p_item->lock );
+    uri = qfu( p_item->psz_uri );
+    vlc_mutex_unlock( &p_item->lock );
+    return uri;
+}
+
+QString PLModel::getTitle( const QModelIndex &index ) const
+{
+    QString title;
+    input_item_t *p_item = getItem( index )->inputItem();
+    char *fb_name = input_item_GetTitle( p_item );
+    if( EMPTY_STR( fb_name ) )
+    {
+        free( fb_name );
+        fb_name = input_item_GetName( p_item );
+    }
+    title = qfu(fb_name);
+    free(fb_name);
+    return title;
+}
+
+bool PLModel::isCurrentItem( const QModelIndex &index, playLocation where ) const
+{
+    if ( where == IN_PLAYLIST )
+    {
+        return itemId( index ) == THEPL->p_playing->i_id;
+    }
+    else if ( where == IN_MEDIALIBRARY )
+    {
+        return THEPL->p_media_library &&
+                itemId( index ) == THEPL->p_media_library->i_id;
+    }
+    return false;
+}
+
 QVariant PLModel::headerData( int section, Qt::Orientation orientation,
                               int role ) const
 {
@@ -448,7 +518,7 @@ QModelIndex PLModel::index( const int row, const int column, const QModelIndex &
 {
     PLItem *parentItem = parent.isValid() ? getItem( parent ) : rootItem;
 
-    PLItem *childItem = parentItem->child( row );
+    PLItem *childItem = static_cast<PLItem*>(parentItem->child( row ));
     if( childItem )
         return createIndex( row, column, childItem );
     else
@@ -460,13 +530,24 @@ QModelIndex PLModel::index( const int i_id, const int c )
     return index( findById( rootItem, i_id ), c );
 }
 
+QModelIndex PLModel::rootIndex() const
+{
+    return index( findById( rootItem, rootItem->id() ), 0 );
+}
+
+bool PLModel::isTree() const
+{
+    return ( ( rootItem && rootItem->id() != p_playlist->p_playing->i_id )
+             || var_InheritBool( p_intf, "playlist-tree" ) );
+}
+
 /* Return the index of a given item */
 QModelIndex PLModel::index( PLItem *item, int column ) const
 {
     if( !item ) return QModelIndex();
-    const PLItem *parent = item->parent();
+    AbstractPLItem *parent = item->parent();
     if( parent )
-        return createIndex( parent->children.lastIndexOf( item ),
+        return createIndex( parent->lastIndexOf( item ),
                             column, item );
     return QModelIndex();
 }
@@ -486,57 +567,26 @@ QModelIndex PLModel::parent( const QModelIndex &index ) const
     PLItem *childItem = getItem( index );
     if( !childItem )
     {
-        msg_Err( p_playlist, "NULL CHILD" );
+        msg_Err( p_playlist, "Item not found" );
         return QModelIndex();
     }
 
-    PLItem *parentItem = childItem->parent();
+    PLItem *parentItem = static_cast<PLItem*>(childItem->parent());
     if( !parentItem || parentItem == rootItem ) return QModelIndex();
     if( !parentItem->parent() )
     {
-        msg_Err( p_playlist, "No parent parent, trying row 0 " );
-        msg_Err( p_playlist, "----- PLEASE REPORT THIS ------" );
+        msg_Err( p_playlist, "No parent found, trying row 0. Please report this" );
         return createIndex( 0, 0, parentItem );
     }
     return createIndex(parentItem->row(), 0, parentItem);
 }
 
-int PLModel::columnCount( const QModelIndex &) const
-{
-    return columnFromMeta( COLUMN_END );
-}
-
 int PLModel::rowCount( const QModelIndex &parent ) const
 {
-    const PLItem *parentItem = parent.isValid() ? getItem( parent ) : rootItem;
+    PLItem *parentItem = parent.isValid() ? getItem( parent ) : rootItem;
     return parentItem->childCount();
 }
 
-QStringList PLModel::selectedURIs()
-{
-    QStringList lst;
-    for( int i = 0; i < current_selection.count(); i++ )
-    {
-        const PLItem *item = getItem( current_selection[i] );
-        if( item )
-        {
-            PL_LOCK;
-            playlist_item_t *p_item = playlist_ItemGetById( p_playlist, item->i_id );
-            if( p_item )
-            {
-                char *psz = input_item_GetURI( p_item->p_input );
-                if( psz )
-                {
-                    lst.append( qfu(psz) );
-                    free( psz );
-                }
-            }
-            PL_UNLOCK;
-        }
-    }
-    return lst;
-}
-
 /************************* Lookups *****************************/
 PLItem *PLModel::findById( PLItem *root, int i_id ) const
 {
@@ -559,18 +609,19 @@ PLItem * PLModel::findInner( PLItem *root, int i_id, bool b_input ) const
     else if( b_input && root->inputItem()->i_id == i_id )
         return root;
 
-    QList<PLItem *>::iterator it = root->children.begin();
+    QList<AbstractPLItem *>::iterator it = root->children.begin();
     while ( it != root->children.end() )
     {
-        if( !b_input && (*it)->id() == i_id )
-            return (*it);
+        PLItem *item = static_cast<PLItem *>(*it);
+        if( !b_input && item->id() == i_id )
+            return item;
 
-        else if( b_input && (*it)->inputItem()->i_id == i_id )
-            return (*it);
+        else if( b_input && item->inputItem()->i_id == i_id )
+            return item;
 
-        if( (*it)->childCount() )
+        if( item->childCount() )
         {
-            PLItem *childFound = findInner( (*it), i_id, b_input );
+            PLItem *childFound = findInner( item, i_id, b_input );
             if( childFound )
                 return childFound;
         }
@@ -597,11 +648,9 @@ bool PLModel::canEdit() const
 void PLModel::processInputItemUpdate( input_thread_t *p_input )
 {
     if( !p_input ) return;
-    if( p_input && !( p_input->b_dead || !vlc_object_alive( p_input ) ) )
-    {
-        PLItem *item = findByInput( rootItem, input_GetItem( p_input )->i_id );
-        if( item ) emit currentIndexChanged( index( item, 0 ) );
-    }
+
+    PLItem *item = findByInput( rootItem, input_GetItem( p_input )->i_id );
+    if( item ) emit currentIndexChanged( index( item, 0 ) );
     processInputItemUpdate( input_GetItem( p_input ) );
 }
 
@@ -619,6 +668,85 @@ void PLModel::processItemRemoval( int i_id )
     removeItem( i_id );
 }
 
+void PLModel::commitBufferedRowInserts()
+{
+    PLItem *toemit = NULL;
+    insertBufferCommitTimer.stop();
+    insertBufferMutex.lock();
+    if ( !insertBuffer.isEmpty() )
+    {
+        beginInsertRows( index( insertBufferRoot, 0 ), insertbuffer_firstrow, insertbuffer_lastrow );
+        foreach (PLItem *item, insertBuffer)
+        {
+            insertBufferRoot->insertChild( item, insertbuffer_firstrow++ );
+            if( item->inputItem() == THEMIM->currentInputItem() )
+                toemit = item;
+        }
+        endInsertRows();
+        insertBuffer.clear();
+    }
+    insertBufferMutex.unlock();
+    if ( toemit )
+        emit currentIndexChanged( index( toemit, 0 ) );
+}
+
+/*
+    Tries to agregate linear inserts of single row. Sends
+    more efficient updates notifications to views and then
+    avoids the flickering effect.
+*/
+void PLModel::bufferedRowInsert( PLItem *item, PLItem *parent, int pos )
+{
+    insertBufferMutex.lock();
+    if ( ! insertBuffer.isEmpty() )
+    {
+        /* Check if we're doing linear insert */
+        if ( parent != insertBufferRoot || pos != insertbuffer_lastrow + 1 )
+        {
+            insertBufferMutex.unlock();
+            commitBufferedRowInserts();
+            bufferedRowInsert( item, parent, pos );
+            return;
+        }
+    }
+
+    if ( insertBuffer.isEmpty() )
+    {
+        insertBuffer << item;
+        insertBufferRoot = parent;
+        insertbuffer_firstrow = pos;
+        insertbuffer_lastrow = pos;
+    } else {
+        insertBuffer << item;
+        insertbuffer_lastrow++;
+    }
+    insertBufferMutex.unlock();
+
+    /* Schedule commit */
+    if ( ! insertBufferCommitTimer.isActive() )
+    {
+        insertBufferCommitTimer.setSingleShot( true );
+        insertBufferCommitTimer.start( 100 );
+    }
+}
+
+bool PLModel::isBufferedForInsert( PLItem *parent, int i_item )
+{
+    bool b_return = false;
+    insertBufferMutex.lock();
+    if ( parent == insertBufferRoot )
+    {
+        foreach (PLItem *item, insertBuffer)
+            if ( item->i_id == i_item )
+            {
+                b_return = true;
+                break;
+            }
+    }
+    insertBufferMutex.unlock();
+    return b_return;
+}
+
 void PLModel::processItemAppend( int i_item, int i_parent )
 {
     playlist_item_t *p_item = NULL;
@@ -627,11 +755,17 @@ void PLModel::processItemAppend( int i_item, int i_parent )
 
     /* Find the Parent */
     PLItem *nodeParentItem = findById( rootItem, i_parent );
+    if( !nodeParentItem )
+    { /* retry as it might have been in buffer */
+        commitBufferedRowInserts();
+        nodeParentItem = findById( rootItem, i_parent );
+    }
     if( !nodeParentItem ) return;
 
     /* Search for an already matching children */
-    foreach( const PLItem *existing, nodeParentItem->children )
-        if( existing->i_id == i_item ) return;
+    if ( isBufferedForInsert( nodeParentItem, i_item ) ) return;
+    foreach( const AbstractPLItem *existing, nodeParentItem->children )
+        if( existing->id() == i_item ) return;
 
     /* Find the child */
     PL_LOCK;
@@ -648,12 +782,7 @@ void PLModel::processItemAppend( int i_item, int i_parent )
     PL_UNLOCK;
 
     /* We insert the newItem (children) inside the parent */
-    beginInsertRows( index( nodeParentItem, 0 ), pos, pos );
-    nodeParentItem->insertChild( newItem, pos );
-    endInsertRows();
-
-    if( newItem->inputItem() == THEMIM->currentInputItem() )
-        emit currentIndexChanged( index( newItem, 0 ) );
+    bufferedRowInsert( newItem, nodeParentItem, pos );
 
     if( latestSearch.isEmpty() ) return;
     search( latestSearch, index( rootItem, 0), false /*FIXME*/ );
@@ -661,10 +790,13 @@ void PLModel::processItemAppend( int i_item, int i_parent )
 
 void PLModel::rebuild( playlist_item_t *p_root )
 {
+    commitBufferedRowInserts();
     /* Invalidate cache */
     i_cached_id = i_cached_input_id = -1;
 
-    if( rootItem ) rootItem->removeChildren();
+    beginResetModel();
+
+    if( rootItem ) rootItem->clearChildren();
 
     PL_LOCK;
     if( p_root ) // Can be NULL
@@ -678,17 +810,17 @@ void PLModel::rebuild( playlist_item_t *p_root )
     PL_UNLOCK;
 
     /* And signal the view */
-    reset();
-
+    endResetModel();
     if( p_root ) emit rootIndexChanged();
 }
 
 void PLModel::takeItem( PLItem *item )
 {
+    commitBufferedRowInserts();
     assert( item );
-    PLItem *parent = item->parent();
+    PLItem *parent = static_cast<PLItem*>(item->parent());
     assert( parent );
-    int i_index = parent->children.indexOf( item );
+    int i_index = parent->indexOf( item );
 
     beginRemoveRows( index( parent, 0 ), i_index, i_index );
     parent->takeChildAt( i_index );
@@ -697,10 +829,10 @@ void PLModel::takeItem( PLItem *item )
 
 void PLModel::insertChildren( PLItem *node, QList<PLItem*>& items, int i_pos )
 {
+    commitBufferedRowInserts();
     assert( node );
     int count = items.count();
     if( !count ) return;
-    printf( "Here I am\n");
     beginInsertRows( index( node, 0 ), i_pos, i_pos + count - 1 );
     for( int i = 0; i < count; i++ )
     {
@@ -713,13 +845,14 @@ void PLModel::insertChildren( PLItem *node, QList<PLItem*>& items, int i_pos )
 void PLModel::removeItem( PLItem *item )
 {
     if( !item ) return;
+    commitBufferedRowInserts();
 
     i_cached_id = -1;
     i_cached_input_id = -1;
 
     if( item->parent() ) {
-        int i = item->parent()->children.indexOf( item );
-        beginRemoveRows( index( item->parent(), 0), i, i );
+        int i = item->parent()->indexOf( item );
+        beginRemoveRows( index( static_cast<PLItem*>(item->parent()), 0), i, i );
         item->parent()->children.removeAt(i);
         delete item;
         endRemoveRows();
@@ -757,7 +890,7 @@ void PLModel::updateChildren( playlist_item_t *p_node, PLItem *root )
 void PLModel::updateTreeItem( PLItem *item )
 {
     if( !item ) return;
-    emit dataChanged( index( item, 0 ) , index( item, columnCount( QModelIndex() ) ) );
+    emit dataChanged( index( item, 0 ) , index( item, columnCount( QModelIndex() ) - 1 ) );
 }
 
 /************************* Actions ******************************/
@@ -789,11 +922,11 @@ void PLModel::doDelete( QModelIndexList selected )
     }
 }
 
-void PLModel::recurseDelete( QList<PLItem*> children, QModelIndexList *fullList )
+void PLModel::recurseDelete( QList<AbstractPLItem*> children, QModelIndexList *fullList )
 {
     for( int i = children.count() - 1; i >= 0 ; i-- )
     {
-        PLItem *item = children[i];
+        PLItem *item = static_cast<PLItem *>(children[i]);
         if( item->childCount() )
             recurseDelete( item->children, fullList );
         fullList->removeAll( index( item, 0 ) );
@@ -803,24 +936,30 @@ void PLModel::recurseDelete( QList<PLItem*> children, QModelIndexList *fullList
 /******* Volume III: Sorting and searching ********/
 void PLModel::sort( const int column, Qt::SortOrder order )
 {
-    sort( rootItem->id(), column, order );
+    sort( QModelIndex(), index( rootItem->id(), 0 ) , column, order );
 }
 
-void PLModel::sort( const int i_root_id, const int column, Qt::SortOrder order )
+void PLModel::sort( QModelIndex caller, QModelIndex rootIndex, const int column, Qt::SortOrder order )
 {
     msg_Dbg( p_intf, "Sorting by column %i, order %i", column, order );
 
     int meta = columnToMeta( column );
     if( meta == COLUMN_END ) return;
 
-    PLItem *item = findById( rootItem, i_root_id );
+    PLItem *item = ( rootIndex.isValid() ) ? getItem( rootIndex )
+                                           : rootItem;
     if( !item ) return;
+
+    int i_root_id = item->id();
+
+    commitBufferedRowInserts();
+
     QModelIndex qIndex = index( item, 0 );
     int count = item->childCount();
     if( count )
     {
         beginRemoveRows( qIndex, 0, count - 1 );
-        item->removeChildren();
+        item->clearChildren();
         endRemoveRows( );
     }
 
@@ -847,13 +986,8 @@ void PLModel::sort( const int i_root_id, const int column, Qt::SortOrder order )
     }
     PL_UNLOCK;
     /* if we have popup item, try to make sure that you keep that item visible */
-    if( i_popup_item > -1 )
-    {
-        PLItem *popupitem = findById( rootItem, i_popup_item );
-        if( popupitem ) emit currentIndexChanged( index( popupitem, 0 ) );
-        /* reset i_popup_item as we don't show it as selected anymore anyway */
-        i_popup_item = -1;
-    }
+    if( caller.isValid() ) emit currentIndexChanged( caller );
+
     else if( currentIndex().isValid() ) emit currentIndexChanged( currentIndex() );
 }
 
@@ -861,6 +995,8 @@ void PLModel::search( const QString& search_text, const QModelIndex & idx, bool
 {
     latestSearch = search_text;
 
+    commitBufferedRowInserts();
+
     /** \todo Fire the search with a small delay ? */
     PL_LOCK;
     {
@@ -874,7 +1010,7 @@ void PLModel::search( const QString& search_text, const QModelIndex & idx, bool
             PLItem *searchRoot = getItem( idx );
 
             beginRemoveRows( idx, 0, searchRoot->childCount() - 1 );
-            searchRoot->removeChildren();
+            searchRoot->clearChildren();
             endRemoveRows();
 
             beginInsertRows( idx, 0, searchRoot->childCount() - 1 );
@@ -902,256 +1038,87 @@ void PLModel::clearPlaylist()
     doDelete(l);
 }
 
-/*********** Popup *********/
-bool PLModel::popup( const QModelIndex & index, const QPoint &point, const QModelIndexList &list )
+void PLModel::ensureArtRequested( const QModelIndex &index )
 {
-    int i_id = index.isValid() ? itemId( index ) : rootItem->id();
-
-    PL_LOCK;
-    playlist_item_t *p_item = playlist_ItemGetById( p_playlist, i_id );
-    if( !p_item )
+    if ( index.isValid() && hasChildren( index ) )
     {
-        PL_UNLOCK;
-        return false;
-    }
-
-    i_popup_item   = index.isValid() ? p_item->i_id : -1;
-    i_popup_parent = index.isValid() ?
-        ( p_item->p_parent ? p_item->p_parent->i_id : -1 ) :
-        ( rootItem->id() );
-
-    bool tree = ( rootItem && rootItem->id() != p_playlist->p_playing->i_id ) ||
-                var_InheritBool( p_intf, "playlist-tree" );
-
-    input_item_t *p_input = p_item->p_input;
-    vlc_gc_incref( p_input );
-    PL_UNLOCK;
-
-    /* */
-    QMenu menu;
-
-    /* Play/Stream/Info static actions */
-    if( i_popup_item > -1 )
-    {
-        menu.addAction( QIcon( ":/menu/play" ), qtr(I_POP_PLAY), this, SLOT( popupPlay() ) );
-        menu.addAction( QIcon( ":/menu/stream" ),
-                        qtr(I_POP_STREAM), this, SLOT( popupStream() ) );
-        menu.addAction( qtr(I_POP_SAVE), this, SLOT( popupSave() ) );
-        menu.addAction( QIcon( ":/menu/info" ), qtr(I_POP_INFO), this, SLOT( popupInfo() ) );
-        if( p_input->psz_uri && !strncasecmp( p_input->psz_uri, "file://", 7 ) )
+        int i_art_policy = var_GetInteger( p_playlist, "album-art" );
+        if ( i_art_policy != ALBUM_ART_ALL ) return;
+        int nbnodes = rowCount( index );
+        QModelIndex child;
+        for( int row = 0 ; row < nbnodes ; row++ )
         {
-            menu.addAction( QIcon( ":/type/folder-grey" ),
-                            qtr( I_POP_EXPLORE ), this, SLOT( popupExplore() ) );
-        }
-        menu.addSeparator();
-    }
-    vlc_gc_decref( p_input );
-
-    /* In PL or ML, allow to add a file/folder */
-    if( canEdit() )
-    {
-        QIcon addIcon( ":/buttons/playlist/playlist_add" );
-        menu.addSeparator();
-        if( tree ) menu.addAction( addIcon, qtr(I_POP_NEWFOLDER), this, SLOT( popupAddNode() ) );
-        if( rootItem->id() == THEPL->p_playing->i_id )
-        {
-            menu.addAction( addIcon, qtr(I_PL_ADDF), THEDP, SLOT( simplePLAppendDialog()) );
-            menu.addAction( addIcon, qtr(I_PL_ADDDIR), THEDP, SLOT( PLAppendDir()) );
-            menu.addAction( addIcon, qtr(I_OP_ADVOP), THEDP, SLOT( PLAppendDialog()) );
-        }
-        else if( THEPL->p_media_library &&
-                    rootItem->id() == THEPL->p_media_library->i_id )
-        {
-            menu.addAction( addIcon, qtr(I_PL_ADDF), THEDP, SLOT( simpleMLAppendDialog()) );
-            menu.addAction( addIcon, qtr(I_PL_ADDDIR), THEDP, SLOT( MLAppendDir() ) );
-            menu.addAction( addIcon, qtr(I_OP_ADVOP), THEDP, SLOT( MLAppendDialog() ) );
+            child = index.child( row, 0 );
+            if ( child.isValid() && getArtUrl( child ).isEmpty() )
+                THEMIM->getIM()->requestArtUpdate( getItem( child )->inputItem() );
         }
     }
-
-    /* Item removal */
-    if( i_popup_item > -1 )
-    {
-        if( rootItem->id() != THEPL->p_playing->i_id )
-            menu.addAction( qtr( "Add to playlist"), this, SLOT( popupAddToPlaylist() ) );
-        menu.addAction( QIcon( ":/buttons/playlist/playlist_remove" ),
-                        qtr(I_POP_DEL), this, SLOT( popupDel() ) );
-    }
-
-    menu.addSeparator();
-
-    if( canEdit() ) {
-        menu.addAction( QIcon( ":/toolbar/clear" ), qtr("Clear playlist"),
-                        this, SLOT( clearPlaylist() ) );
-    }
-
-    /* Playlist sorting */
-    if( !sortingMenu )
-    {
-        sortingMenu = new QMenu( qtr( "Sort by" ) );
-        sortingMapper = new QSignalMapper( this );
-        /* Choose what columns to show in sorting menu, not sure if this should be configurable*/
-        QList<int> sortingColumns;
-        sortingColumns << COLUMN_TITLE << COLUMN_ARTIST << COLUMN_ALBUM << COLUMN_TRACK_NUMBER << COLUMN_URI;
-        foreach( int Column, sortingColumns )
-        {
-            QAction *asc  = sortingMenu->addAction( qfu( psz_column_title( Column ) ) + " " + qtr("Ascending") );
-            QAction *desc = sortingMenu->addAction( qfu( psz_column_title( Column ) ) + " " + qtr("Descending") );
-            sortingMapper->setMapping( asc, columnFromMeta(Column) + 1 );
-            sortingMapper->setMapping( desc, -1 * (columnFromMeta(Column)+1) );
-            CONNECT( asc, triggered(), sortingMapper, map() );
-            CONNECT( desc, triggered(), sortingMapper, map() );
-        }
-        CONNECT( sortingMapper, mapped( int ), this, popupSort( int ) );
-    }
-    menu.addMenu( sortingMenu );
-
-    /* Zoom */
-    QMenu *zoomMenu = new QMenu( qtr( "Display size" ) );
-    zoomMenu->addAction( qtr( "Increase" ), this, SLOT( increaseZoom() ) );
-    zoomMenu->addAction( qtr( "Decrease" ), this, SLOT( decreaseZoom() ) );
-    menu.addMenu( zoomMenu );
-
-    /* Store the current selected item for popup*() methods */
-    current_selection = list;
-
-    /* Display and forward the result */
-    if( !menu.isEmpty() )
-    {
-        menu.exec( point ); return true;
-    }
-    else return false;
 }
 
-void PLModel::popupDel()
+
+void PLModel::createNode( QModelIndex index, QString name )
 {
-    doDelete( current_selection );
-}
-
-void PLModel::popupPlay()
-{
-    PL_LOCK;
-    {
-        playlist_item_t *p_item = playlist_ItemGetById( p_playlist,
-                                                        i_popup_item );
-        activateItem( p_item );
-    }
-    PL_UNLOCK;
-}
-
-void PLModel::popupAddToPlaylist()
-{
-    playlist_Lock( THEPL );
-
-    foreach( QModelIndex currentIndex, current_selection )
-    {
-        /* Don't add item multiple times when there are more columns in the model */
-        if( currentIndex.column() )
-            continue;
-
-        playlist_item_t *p_item = playlist_ItemGetById( THEPL, itemId( currentIndex ) );
-        if( !p_item ) continue;
-
-        playlist_NodeAddCopy( THEPL, p_item,
-                              THEPL->p_playing,
-                              PLAYLIST_END );
-    }
-    playlist_Unlock( THEPL );
-}
-
-void PLModel::popupInfo()
-{
-    PL_LOCK;
-    playlist_item_t *p_item = playlist_ItemGetById( p_playlist,
-                                                    i_popup_item );
-    if( p_item )
-    {
-        input_item_t* p_input = p_item->p_input;
-        vlc_gc_incref( p_input );
-        PL_UNLOCK;
-        MediaInfoDialog *mid = new MediaInfoDialog( p_intf, p_input );
-        vlc_gc_decref( p_input );
-        mid->setParent( PlaylistDialog::getInstance( p_intf ),
-                        Qt::Dialog );
-        mid->show();
-    } else
-        PL_UNLOCK;
-}
-
-void PLModel::popupStream()
-{
-    QStringList mrls = selectedURIs();
-    if( !mrls.isEmpty() )
-        THEDP->streamingDialog( NULL, mrls[0], false );
-}
-
-void PLModel::popupSave()
-{
-    QStringList mrls = selectedURIs();
-    if( !mrls.isEmpty() )
-        THEDP->streamingDialog( NULL, mrls[0] );
-}
-
-void PLModel::popupExplore()
-{
-    char *uri = NULL, *path = NULL;
+    if( name.isEmpty() || !index.isValid() ) return;
 
     PL_LOCK;
-    playlist_item_t *p_item = playlist_ItemGetById( p_playlist, i_popup_item );
-    if( p_item )
-    {
-        input_item_t *p_input = p_item->p_input;
-        uri = input_item_GetURI( p_input );
-    }
-    PL_UNLOCK;
-
-    if( uri != NULL )
-    {
-        path = make_path( uri );
-        free( uri );
-    }
-    if( path == NULL )
-        return;
-
-    QFileInfo info( qfu( path ) );
-    free( path );
-
-    QDesktopServices::openUrl( QUrl::fromLocalFile( info.absolutePath() ) );
-}
-
-void PLModel::popupAddNode()
-{
-    bool ok;
-    QString name = QInputDialog::getText( PlaylistDialog::getInstance( p_intf ),
-        qtr( I_NEW_DIR ), qtr( I_NEW_DIR_NAME ),
-        QLineEdit::Normal, QString(), &ok);
-    if( !ok || name.isEmpty() ) return;
-
-    PL_LOCK;
-    playlist_item_t *p_item = playlist_ItemGetById( p_playlist,
-                                                    i_popup_parent );
+    index = index.parent();
+    if ( !index.isValid() ) index = rootIndex();
+    playlist_item_t *p_item = playlist_ItemGetById( p_playlist, itemId( index ) );
     if( p_item )
         playlist_NodeCreate( p_playlist, qtu( name ), p_item, PLAYLIST_END, 0, NULL );
     PL_UNLOCK;
 }
 
-void PLModel::popupSort( int column )
+void PLModel::actionSlot( QAction *action )
 {
-    sort( i_popup_parent,
-          column > 0 ? column - 1 : -column - 1,
-          column > 0 ? Qt::AscendingOrder : Qt::DescendingOrder );
-}
+    QString name;
+    QStringList mrls;
+    QModelIndex index;
 
-/* */
-void PLModel::increaseZoom()
-{
-    i_zoom++;
-    emit layoutChanged();
-}
+    actionsContainerType a = action->data().value<actionsContainerType>();
+    switch ( a.action )
+    {
 
-void PLModel::decreaseZoom()
-{
-    i_zoom--;
-    emit layoutChanged();
+    case actionsContainerType::ACTION_PLAY:
+        PL_LOCK;
+        {
+            if ( a.indexes.first().isValid() )
+            {
+                playlist_item_t *p_item = playlist_ItemGetById( p_playlist,
+                                             itemId( a.indexes.first() ) );
+                activateItem( p_item );
+            }
+        }
+        PL_UNLOCK;
+        break;
+
+    case actionsContainerType::ACTION_ADDTOPLAYLIST:
+        PL_LOCK;
+        foreach( QModelIndex currentIndex, a.indexes )
+        {
+            playlist_item_t *p_item = playlist_ItemGetById( THEPL, itemId( currentIndex ) );
+            if( !p_item ) continue;
+
+            playlist_NodeAddCopy( THEPL, p_item,
+                                  THEPL->p_playing,
+                                  PLAYLIST_END );
+        }
+        PL_UNLOCK;
+        break;
+
+    case actionsContainerType::ACTION_REMOVE:
+        doDelete( a.indexes );
+        break;
+
+    case actionsContainerType::ACTION_SORT:
+        index = a.indexes.first().parent();
+        if( !index.isValid() ) index = rootIndex();
+        sort( a.indexes.first(), index,
+              a.column > 0 ? a.column - 1 : -a.column - 1,
+              a.column > 0 ? Qt::AscendingOrder : Qt::DescendingOrder );
+        break;
+
+    }
 }
 
 /******************* Drag and Drop helper class ******************/
