@@ -4,7 +4,7 @@
  * Copyright (C) 2007-2009 the VideoLAN team
  * Copyright (C) 2007 Société des arts technologiques
  * Copyright (C) 2007 Savoir-faire Linux
- * $Id: 10a24cbc9e9ad6a6de1abb892e63f4f5f89dc222 $
+ * $Id: 8b0fc1281e7f557201deaa4136e2687a279ddc70 $
  *
  * Authors: Jean-Baptiste Kempf <jb@videolan.org>
  *          Pierre-Luc Beaudoin <pierre-luc.beaudoin@savoirfairelinux.com>
@@ -34,6 +34,7 @@
 #include <QLabel>
 #include <QLineEdit>
 #include <QFileDialog>
+#include <QUrl>
 
 #define I_FILE_SLASH_DIR \
     I_DIR_OR_FOLDER( N_("File/Directory"), N_("File/Folder") )
@@ -68,15 +69,10 @@ SoutInputBox::SoutInputBox( QWidget *_parent, const QString& mrl ) : QGroupBox(
 
 void SoutInputBox::setMRL( const QString& mrl )
 {
-    sourceLine->setText( mrl );
-    QString type;
-    int i = mrl.indexOf( "://" );
-    if( i != -1 )
-    {
-        type = mrl.left( i );
-    }
-    else
-        type = qtr( I_FILE_SLASH_DIR );
+    QUrl uri = QUrl::fromEncoded( mrl.toLatin1() );
+    sourceLine->setText( uri.toString() );
+    QString type = uri.scheme();
+    if ( type.isEmpty() ) type = qtr( I_FILE_SLASH_DIR );
     sourceValueLabel->setText( type );
 }
 
@@ -108,8 +104,7 @@ FileDestBox::FileDestBox( QWidget *_parent ) : VirtualDestBox( _parent )
     BUTTONACT( fileSelectButton, fileBrowse() );
 }
 
-QString FileDestBox::getMRL( const QString& mux, const int, const bool,
-                             const QString&, const QString& )
+QString FileDestBox::getMRL( const QString& mux )
 {
     if( fileEdit->text().isEmpty() ) return "";
 
@@ -129,6 +124,7 @@ QString FileDestBox::getMRL( const QString& mux, const int, const bool,
         }
     }
     m.option( "dst", outputfile );
+    m.option( "no-overwrite" );
     m.end();
 
     return m.getMrl();
@@ -174,8 +170,7 @@ HTTPDestBox::HTTPDestBox( QWidget *_parent ) : VirtualDestBox( _parent )
     CT( HTTPEdit );
 }
 
-QString HTTPDestBox::getMRL( const QString& mux, const int, const bool,
-                             const QString&, const QString& )
+QString HTTPDestBox::getMRL( const QString& mux )
 {
     if( HTTPEdit->text().isEmpty() ) return "";
 
@@ -234,8 +229,7 @@ MMSHDestBox::MMSHDestBox( QWidget *_parent ) : VirtualDestBox( _parent )
     CT( MMSHEdit );
 }
 
-QString MMSHDestBox::getMRL( const QString&, const int, const bool,
-                             const QString&, const QString& )
+QString MMSHDestBox::getMRL( const QString& )
 {
     if( MMSHEdit->text().isEmpty() ) return "";
 
@@ -280,8 +274,7 @@ RTSPDestBox::RTSPDestBox( QWidget *_parent ) : VirtualDestBox( _parent )
     CT( RTSPEdit );
 }
 
-QString RTSPDestBox::getMRL( const QString&, const int, const bool,
-                             const QString&, const QString& )
+QString RTSPDestBox::getMRL( const QString& )
 {
     if( RTSPEdit->text().isEmpty() ) return "";
 
@@ -330,34 +323,16 @@ UDPDestBox::UDPDestBox( QWidget *_parent ) : VirtualDestBox( _parent )
     CT( UDPEdit );
 }
 
-QString UDPDestBox::getMRL( const QString& mux, const int ttl, const bool sap,
-                            const QString& sapName, const QString& sapGroup )
+QString UDPDestBox::getMRL( const QString& mux )
 {
     if( UDPEdit->text().isEmpty() ) return "";
 
     SoutMrl m;
-    m.begin( "std" );
-
-    SoutMrl access;
-    access.begin( "udp" );
-    access.option( "ttl", ttl );
-    access.end();
-    m.option( "access", access.getMrl() );
- 
+    m.begin( "udp" );
     /* udp output, ts-mux is really only reasonable one to use*/
     if( !mux.isEmpty() && !mux.compare("ts" ) )
         m.option( "mux", mux );
     m.option( "dst", UDPEdit->text(), UDPPort->value() );
-
-    if( sap )
-    {
-        m.option( "sap" );
-        if( !sapName.isEmpty() )
-            m.option( "name", sapName );
-        if( !sapGroup.isEmpty() )
-            m.option( "group", sapGroup );
-    }
-   
     m.end();
 
     return m.getMrl();
@@ -366,7 +341,7 @@ QString UDPDestBox::getMRL( const QString& mux, const int ttl, const bool sap,
 
 
 RTPDestBox::RTPDestBox( QWidget *_parent, const char *_mux )
-    : VirtualDestBox( _parent ), mux( _mux )
+    : VirtualDestBox( _parent ), mux( qfu(_mux) )
 {
     QGridLayout *layout = new QGridLayout( this );
 
@@ -376,45 +351,49 @@ RTPDestBox::RTPDestBox( QWidget *_parent, const char *_mux )
     layout->addWidget(rtpOutput, 0, 0, 1, -1);
 
     QLabel *RTPLabel = new QLabel( qtr("Address"), this );
-    QLabel *RTPPortLabel = new QLabel( qtr("Base port"), this );
-    layout->addWidget(RTPLabel, 1, 0, 1, 1);
-    layout->addWidget(RTPPortLabel, 2, 0, 1, 1);
-
     RTPEdit = new QLineEdit(this);
+    layout->addWidget(RTPLabel, 1, 0, 1, 1);
+    layout->addWidget(RTPEdit, 1, 1, 1, 1);
 
+    QLabel *RTPPortLabel = new QLabel( qtr("Base port"), this );
     RTPPort = new QSpinBox(this);
     RTPPort->setMaximumSize(QSize(90, 16777215));
     RTPPort->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
     RTPPort->setMinimum(1);
     RTPPort->setMaximum(65535);
     RTPPort->setValue(5004);
-
-    layout->addWidget(RTPEdit, 1, 1, 1, 1);
+    layout->addWidget(RTPPortLabel, 2, 0, 1, 1);
     layout->addWidget(RTPPort, 2, 1, 1, 1);
 
-    CS( RTPPort );
+    QLabel *SAPNameLabel = new QLabel( qtr("Stream name"), this );
+    SAPName = new QLineEdit(this);
+    layout->addWidget(SAPNameLabel, 3, 0, 1, 1);
+    layout->addWidget(SAPName, 3, 1, 1, 1);
+
     CT( RTPEdit );
+    CS( RTPPort );
+    CT( SAPName );
 }
 
-QString RTPDestBox::getMRL( const QString&, const int ttl, const bool sap,
-                            const QString& sapName, const QString& )
+QString RTPDestBox::getMRL( const QString& )
 {
-    if( RTPEdit->text().isEmpty() ) return "";
+    QString addr = RTPEdit->text();
+    QString name = SAPName->text();
+
+    if( addr.isEmpty() ) return qfu("");
 
     SoutMrl m;
     m.begin( "rtp" );
     m.option( "dst", RTPEdit->text() );
     m.option( "port", RTPPort->value() );
     /* mp4-mux ain't usable in rtp-output either */
-    if( mux != NULL )
-        m.option( "mux", qfu( mux ) );
-    if( sap )
+    if( !mux.isEmpty() )
+        m.option( "mux", mux );
+    if( !name.isEmpty() )
     {
         m.option( "sap" );
-        if( !sapName.isEmpty() )
-            m.option( "name", sapName );
+        m.option( "name", name );
     }
-    m.option( "ttl", ttl );
     m.end();
 
     return m.getMrl();
@@ -465,8 +444,7 @@ ICEDestBox::ICEDestBox( QWidget *_parent ) : VirtualDestBox( _parent )
 #undef CS
 #undef CT
 
-QString ICEDestBox::getMRL( const QString&, const int, const bool,
-                            const QString&, const QString& )
+QString ICEDestBox::getMRL( const QString& )
 {
     if( ICEEdit->text().isEmpty() ) return "";
 
