@@ -2,7 +2,7 @@
  * vlm.cpp : VLM Management
  ****************************************************************************
  * Copyright © 2008 the VideoLAN team
- * $Id: a4e27361b4b8e201a5bd3b1ab06d3dd612947660 $
+ * $Id: bc7d96bd9ded351780feb9201915d6f90f8cae92 $
  *
  * Authors: Jean-Baptiste Kempf <jb@videolan.org>
  *          Jean-François Massol <jf.massol -at- gmail.com>
@@ -94,7 +94,7 @@ VLMDialog::VLMDialog( intf_thread_t *_p_intf ) : QVLCDialog( (QWidget*)_p_intf->
     date = new QDateTimeEdit( QDate::currentDate() );
     date->setAlignment( Qt::AlignRight );
     date->setCalendarPopup( true );
-#ifdef WIN32
+#ifdef _WIN32
     date->setDisplayFormat( "dd MM yyyy" );
 #else
     date->setDisplayFormat( "dd MMMM yyyy" );
@@ -606,12 +606,12 @@ void VLMBroadcast::stop()
 /****************
  * VLMSchedule
  ****************/
-VLMSchedule::VLMSchedule( const QString& name, const QString& input,
+VLMSchedule::VLMSchedule( const QString& name_, const QString& input,
                           const QString& inputOptions,
                           const QString& output, QDateTime _schetime,
                           QDateTime _schedate, int _scherepeatnumber,
                           int _repeatDays, bool enabled, VLMDialog *parent )
-            : VLMAWidget( name, input, inputOptions, output, enabled, parent,
+            : VLMAWidget( name_, input, inputOptions, output, enabled, parent,
                           QVLM_Schedule )
 {
     nameLabel->setText( qtr("Schedule: ") + name );
@@ -632,10 +632,10 @@ void VLMSchedule::update()
 /****************
  * VLMVOD
  ****************/
-VLMVod::VLMVod( const QString& name, const QString& input,
+VLMVod::VLMVod( const QString& name_, const QString& input,
                 const QString& inputOptions, const QString& output,
                 bool enabled, const QString& _mux, VLMDialog *parent)
-       : VLMAWidget( name, input, inputOptions, output, enabled, parent,
+       : VLMAWidget( name_, input, inputOptions, output, enabled, parent,
                      QVLM_VOD )
 {
     nameLabel->setText( qtr("VOD: ") + name );
@@ -752,7 +752,7 @@ void VLMWrapper::ControlBroadcast( const QString& name, int BroadcastStatus,
         command += " stop";
         break;
     case ControlBroadcastSeek:
-        command += " seek" + seek;
+        command += " seek " + QString::number( seek );
         break;
     }
     vlm_ExecuteCommand( p_vlm, qtu( command ), &message );
