@@ -2,7 +2,7 @@
  * menus.hpp : Menus handling
  ****************************************************************************
  * Copyright (C) 2006 the VideoLAN team
- * $Id: 3ccee3b9481037d85a6731543f1b759dc7b22165 $
+ * $Id: 3b39c7b12f72104bd9877d70a1077efc86c3fc1e $
  *
  * Authors: Clément Stenac <zorglub@videolan.org>
  *          Jean-Baptiste Kempf <jb@videolan.org>
@@ -103,8 +103,9 @@ private:
     /* All main Menus */
     static QMenu *FileMenu( intf_thread_t *, QWidget *, MainInterface * mi = NULL );
 
-    static QMenu *ToolsMenu( QMenu * );
-    static QMenu *ToolsMenu( QWidget *parent ) { return ToolsMenu( new QMenu( parent ) ); }
+    static QMenu *ToolsMenu( intf_thread_t *, QMenu * );
+    static QMenu *ToolsMenu( intf_thread_t * p_intf, QWidget *parent )
+        { return ToolsMenu( p_intf, new QMenu( parent ) ); }
 
     static QMenu *ViewMenu( intf_thread_t *, QMenu *, MainInterface * mi = NULL );
 
@@ -117,11 +118,14 @@ private:
     }
     static QMenu *RebuildNavigMenu( intf_thread_t *, QMenu *, bool b_keep = false );
 
-    static QMenu *VideoMenu( intf_thread_t *, QMenu *, bool b_subtitle = true );
+    static QMenu *VideoMenu( intf_thread_t *, QMenu * );
     static QMenu *VideoMenu( intf_thread_t *p_intf, QWidget *parent ) {
         return VideoMenu( p_intf, new QMenu( parent ) );
     }
-    static QMenu *SubtitleMenu( QMenu *current);
+    static QMenu *SubtitleMenu( intf_thread_t *, QMenu *current, bool b_popup = false );
+    static QMenu *SubtitleMenu( intf_thread_t *p_intf, QWidget *parent) {
+        return SubtitleMenu( p_intf, new QMenu( parent ) );
+    }
 
     static QMenu *AudioMenu( intf_thread_t *, QMenu * );
     static QMenu *AudioMenu( intf_thread_t *p_intf, QWidget *parent ) {
@@ -150,7 +154,9 @@ private:
     static void EnableStaticEntries( QMenu *, bool );
 
     /* recentMRL menu */
-    static QMenu *recentsMenu;
+    static QMenu *recentsMenu, *audioDeviceMenu;
+
+    static void updateAudioDevice( intf_thread_t *, audio_output_t *, QMenu* );
 
 public slots:
     static void updateRecents( intf_thread_t * );
@@ -173,6 +179,7 @@ public:
             case 2: VLCMenuBar::VideoMenu( p_intf, menu ); break;
             case 3: VLCMenuBar::RebuildNavigMenu( p_intf, menu ); break;
             case 4: VLCMenuBar::ViewMenu( p_intf, menu ); break;
+            case 5: VLCMenuBar::SubtitleMenu( p_intf, menu ); break;
         }
     }
 private:
