@@ -7,19 +7,19 @@
  * Authors: Christopher Mueller <christopher.mueller@itec.uni-klu.ac.at>
  *          Christian Timmerer  <christian.timmerer@itec.uni-klu.ac.at>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published
  * by the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 #ifndef BASICCMPARSER_H_
@@ -30,7 +30,7 @@
 #include "mpd/IMPDParser.h"
 #include "mpd/MPD.h"
 #include "mpd/Period.h"
-#include "mpd/Group.h"
+#include "mpd/AdaptationSet.h"
 #include "mpd/Representation.h"
 #include "mpd/BaseUrl.h"
 #include "mpd/SegmentInfo.h"
@@ -52,17 +52,17 @@ namespace dash
                 MPD*    getMPD ();
 
             private:
-                void    handleDependencyId( Representation* rep, const Group* group, const std::string& dependencyId );
+                void    handleDependencyId( Representation* rep, const AdaptationSet* adaptationSet, const std::string& dependencyId );
 
             private:
                 bool    setMPD              ();
                 void    setPeriods          (dash::xml::Node *root);
                 void    parseSegmentTimeline( xml::Node* node, SegmentInfoCommon *segmentInfo );
                 void    parseSegmentInfoCommon( xml::Node* node, SegmentInfoCommon *segmentInfo );
-                void    parseSegmentInfoDefault( xml::Node* node, Group* group );
-                void    setGroups           (dash::xml::Node *root, Period *period);
+                void    parseSegmentInfoDefault( xml::Node* node, AdaptationSet* group );
+                void    setAdaptationSet           (dash::xml::Node *root, Period *period);
                 void    parseTrickMode( dash::xml::Node *node, Representation *repr );
-                void    setRepresentations  (dash::xml::Node *root, Group *group);
+                void    setRepresentations  (dash::xml::Node *root, AdaptationSet *group);
                 bool    setSegmentInfo      (dash::xml::Node *root, Representation *rep);
                 void    setInitSegment      (dash::xml::Node *root, SegmentInfoCommon *info);
                 bool    setSegments         (dash::xml::Node *root, SegmentInfo *info );
