@@ -7,39 +7,30 @@
  * Authors: Christopher Mueller <christopher.mueller@itec.uni-klu.ac.at>
  *          Christian Timmerer  <christian.timmerer@itec.uni-klu.ac.at>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published
  * by the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 #ifndef DOMPARSER_H_
 #define DOMPARSER_H_
 
 #include <vlc_common.h>
-#include <vlc_plugin.h>
 #include <vlc_stream.h>
 #include <vlc_xml.h>
 
-#include <string.h>
-#include <vector>
-#include <iostream>
-#include <cstdio>
-#include <fstream>
-#include <stdio.h>
-
-#include "xml/Node.h"
-#include "http/IHTTPConnection.h"
 #include "mpd/IMPDManager.h"
+#include "xml/Node.h"
 
 namespace dash
 {
@@ -55,6 +46,7 @@ namespace dash
                 Node*               getRootNode ();
                 void                print       ();
                 static bool         isDash      (stream_t *stream);
+                mpd::Profile        getProfile  ();
 
             private:
                 Node                *root;
