@@ -2,23 +2,23 @@
  * blend2.cpp: Blend one picture with alpha onto another picture
  *****************************************************************************
  * Copyright (C) 2012 Laurent Aimar
- * $Id: 6c05d68a66d7119685d2e118615b857108646c20 $
+ * $Id: 03c92e5eada2327352d2243d43fe65c43c1b1b29 $
  *
  * Authors: Laurent Aimar <fenrir _AT_ videolan _DOT_ org>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * This program is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  *****************************************************************************/
 
 /*****************************************************************************
@@ -607,6 +607,12 @@ static void Blend(filter_t *filter,
 {
     filter_sys_t *sys = filter->p_sys;
 
+    if( x_offset < 0 || y_offset < 0 )
+    {
+        msg_Err( filter, "Blend cannot process negative offsets" );
+        return;
+    }
+
     int width  = __MIN((int)filter->fmt_out.video.i_visible_width - x_offset,
                        (int)filter->fmt_in.video.i_visible_width);
     int height = __MIN((int)filter->fmt_out.video.i_visible_height - y_offset,
