@@ -2,7 +2,7 @@
  * events.c: Windows DirectX video output events handler
  *****************************************************************************
  * Copyright (C) 2001-2009 VLC authors and VideoLAN
- * $Id: ada4d43090d58d3fd5c18822e18fe8120c87de99 $
+ * $Id: df1697f3f71b0bc28b026810c1d338388a0cb607 $
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *
@@ -225,7 +225,8 @@ static void *EventThread( void *p_this )
     }
 
     /* Prevent monitor from powering off */
-    SetThreadExecutionState( ES_DISPLAY_REQUIRED | ES_SYSTEM_REQUIRED | ES_CONTINUOUS );
+    if (var_GetBool(vd, "disable-screensaver"))
+	SetThreadExecutionState( ES_DISPLAY_REQUIRED | ES_SYSTEM_REQUIRED | ES_CONTINUOUS );
 
     /* Main loop */
     /* GetMessage will sleep if there's no message in the queue */
@@ -753,13 +754,6 @@ static long FAR PASCAL DirectXEventProc( HWND hwnd, UINT message,
     }
     vout_display_t *vd = p_event->vd;
 
-    /* Catch the screensaver and the monitor turn-off */
-    if( message == WM_SYSCOMMAND &&
-        ( (wParam & 0xFFF0) == SC_SCREENSAVE || (wParam & 0xFFF0) == SC_MONITORPOWER ) )
-    {
-        //if( vd ) msg_Dbg( vd, "WinProc WM_SYSCOMMAND screensaver" );
-        return 0; /* this stops them from happening */
-    }
 #if 0
     if( message == WM_SETCURSOR )
     {
