@@ -45,7 +45,7 @@ function fetch_art()
     if not fd then return nil end
     page = fd:read( 65653 )
     fd = nil
-    _, _, arturl = string.find( page, "<img  width=\"174\" src=\"([^\"]+)\" class=\"art\" />\n" )
+    _, _, arturl = string.find( page, "<meta property=\"og:image\" content=\"([^\"]+)\" />" )
     -- Don't use default album-art (not found one)
     if not arturl or string.find( arturl, "default_album_mega.png") then
        return nil
