@@ -27,8 +27,7 @@
 
 #ifdef ENABLE_NLS
 # include <libintl.h>
-# if defined (__APPLE__) || defined (WIN32) || defined(__OS2__)
-#  include "config/configuration.h"
+# if defined (__APPLE__) || defined (_WIN32) || defined(__OS2__)
 #  include <vlc_charset.h>
 # endif
 #endif
@@ -37,7 +36,7 @@ int vlc_bindtextdomain (const char *domain)
 {
 #if defined (ENABLE_NLS)
     /* Specify where to find the locales for current domain */
-# if !defined (__APPLE__) && !defined (WIN32) && !defined(__OS2__)
+# if !defined (__APPLE__) && !defined (_WIN32) && !defined(__OS2__)
     static const char path[] = LOCALEDIR;
 
     if (bindtextdomain (domain, path) == NULL)
@@ -46,7 +45,7 @@ int vlc_bindtextdomain (const char *domain)
         return -1;
     }
 # else
-    char *datadir = config_GetDataDirDefault();
+    char *datadir = config_GetDataDir();
     if (unlikely(datadir == NULL))
         return -1;
 
@@ -98,10 +97,17 @@ int vlc_bindtextdomain (const char *domain)
 char *vlc_gettext (const char *msgid)
 {
 #ifdef ENABLE_NLS
-    if (unlikely(!*msgid))
-        return (char *)"";
-    return dgettext (PACKAGE_NAME, msgid);
-#else
-    return (char *)msgid;
+    if (likely(*msgid))
+        return dgettext (PACKAGE_NAME, msgid);
 #endif
+    return (char *)msgid;
+}
+
+char *vlc_ngettext (const char *msgid, const char *plural, unsigned long n)
+{
+#ifdef ENABLE_NLS
+    if (likely(*msgid))
+        return dngettext (PACKAGE_NAME, msgid, plural, n);
+#endif
+    return (char *)((n == 1) ? msgid : plural);
 }
