@@ -3,7 +3,7 @@
  *****************************************************************************
  * Copyright (C) 2005 VLC authors and VideoLAN
  * Copyright (C) 2002-2007 Rémi Denis-Courmont
- * $Id: 0d5cdf873792f36e0b29574145a777045c73535e $
+ * $Id: 16dd68e3a3e21dc7aaf2c96dba675b112a29ea19 $
  *
  * Author: Rémi Denis-Courmont <rem # videolan.org>
  *
@@ -27,7 +27,6 @@
 #endif
 
 #include <vlc_common.h>
-#include <vlc_charset.h>
 
 #include <stddef.h> /* size_t */
 #include <string.h> /* strlen(), memcpy(), memset(), strchr() */
@@ -72,7 +71,6 @@ int vlc_getnameinfo( const struct sockaddr *sa, int salen,
 /**
  * Resolves a host name to a list of socket addresses (like getaddrinfo()).
  *
- * @param p_this a VLC object
  * @param node host name to resolve (encoded as UTF-8), or NULL
  * @param i_port port number for the socket addresses
  * @param p_hints parameters (see getaddrinfo() manual page)
@@ -81,111 +79,47 @@ int vlc_getnameinfo( const struct sockaddr *sa, int salen,
  * On failure, *res is undefined. On success, it must be freed with
  * freeaddrinfo().
  */
-int vlc_getaddrinfo( vlc_object_t *p_this, const char *node,
-                     int i_port, const struct addrinfo *p_hints,
-                     struct addrinfo **res )
+int vlc_getaddrinfo (const char *node, unsigned port,
+                     const struct addrinfo *hints, struct addrinfo **res)
 {
-    struct addrinfo hints;
-    char psz_buf[NI_MAXHOST], psz_service[6];
+    char hostbuf[NI_MAXHOST], portbuf[6], *servname;
 
     /*
      * In VLC, we always use port number as integer rather than strings
      * for historical reasons (and portability).
      */
-    if( ( i_port > 65535 ) || ( i_port < 0 ) )
+    if (port != 0)
     {
-        msg_Err( p_this, "invalid port number %d specified", i_port );
-        return EAI_SERVICE;
+        if (port > 65535)
+            return EAI_SERVICE;
+        /* cannot overflow */
+        snprintf (portbuf, sizeof (portbuf), "%u", port);
+        servname = portbuf;
     }
-
-    /* cannot overflow */
-    snprintf( psz_service, 6, "%d", i_port );
-
-    /* Check if we have to force ipv4 or ipv6 */
-    memset (&hints, 0, sizeof (hints));
-    if (p_hints != NULL)
-    {
-        const int safe_flags =
-            AI_PASSIVE |
-            AI_CANONNAME |
-            AI_NUMERICHOST |
-            AI_NUMERICSERV |
-#ifdef AI_ALL
-            AI_ALL |
-#endif
-#ifdef AI_ADDRCONFIG
-            AI_ADDRCONFIG |
-#endif
-#ifdef AI_V4MAPPED
-            AI_V4MAPPED |
-#endif
-            0;
-
-        hints.ai_family = p_hints->ai_family;
-        hints.ai_socktype = p_hints->ai_socktype;
-        hints.ai_protocol = p_hints->ai_protocol;
-        /* Unfortunately, some flags chang the layout of struct addrinfo, so
-         * they cannot be copied blindly from p_hints to &hints. Therefore, we
-         * only copy flags that we know for sure are "safe".
-         */
-        hints.ai_flags = p_hints->ai_flags & safe_flags;
-    }
-
-    /* We only ever use port *numbers* */
-    hints.ai_flags |= AI_NUMERICSERV;
+    else
+        servname = NULL;
 
     /*
      * VLC extensions :
-     * - accept "" as NULL
-     * - ignore square brackets
+     * - accept the empty string as unspecified host (i.e. NULL)
+     * - ignore square brackets (for IPv6 numerals)
      */
     if (node != NULL)
     {
         if (node[0] == '[')
         {
             size_t len = strlen (node + 1);
-            if ((len <= sizeof (psz_buf)) && (node[len] == ']'))
+            if ((len <= sizeof (hostbuf)) && (node[len] == ']'))
             {
                 assert (len > 0);
-                memcpy (psz_buf, node + 1, len - 1);
-                psz_buf[len - 1] = '\0';
-                node = psz_buf;
+                memcpy (hostbuf, node + 1, len - 1);
+                hostbuf[len - 1] = '\0';
+                node = hostbuf;
             }
         }
         if (node[0] == '\0')
             node = NULL;
     }
 
-    int ret;
-    node = ToLocale (node);
-#ifdef WIN32
-    /*
-     * Winsock tries to resolve numerical IPv4 addresses as AAAA
-     * and IPv6 addresses as A... There comes the bug-to-bug fix.
-     */
-    if ((hints.ai_flags & AI_NUMERICHOST) == 0)
-    {
-        hints.ai_flags |= AI_NUMERICHOST;
-        ret = getaddrinfo (node, psz_service, &hints, res);
-        if (ret == 0)
-            goto out;
-        hints.ai_flags &= ~AI_NUMERICHOST;
-    }
-#endif
-#ifdef AI_IDN
-    /* Run-time I18n Domain Names support */
-    hints.ai_flags |= AI_IDN;
-    ret = getaddrinfo (node, psz_service, &hints, res);
-    if (ret != EAI_BADFLAGS)
-        goto out;
-    /* IDN not available: disable and retry without it */
-    hints.ai_flags &= ~AI_IDN;
-#endif
-    ret = getaddrinfo (node, psz_service, &hints, res);
-
-#if defined(AI_IDN) || defined(WIN32)
-out:
-#endif
-    LocaleFree (node);
-    return ret;
+    return getaddrinfo (node, servname, hints, res);
 }
