@@ -2,7 +2,7 @@
  * loadsave.c : Playlist loading / saving functions
  *****************************************************************************
  * Copyright (C) 1999-2004 VLC authors and VideoLAN
- * $Id: 59d1a7e40682312622422a99080f478a3a039742 $
+ * $Id: 85a3e9eb8bb54d966038a385b86f574f99f280ec $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -24,6 +24,10 @@
 # include "config.h"
 #endif
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
+
 #include <vlc_common.h>
 #include <vlc_playlist.h>
 #include <vlc_events.h>
@@ -33,10 +37,6 @@
 #include <vlc_url.h>
 #include <vlc_modules.h>
 
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
-
 int playlist_Export( playlist_t * p_playlist, const char *psz_filename,
                      playlist_item_t *p_export_root, const char *psz_type )
 {
@@ -85,7 +85,7 @@ int playlist_Import( playlist_t *p_playlist, const char *psz_file )
 {
     input_item_t *p_input;
     const char *const psz_option = "meta-file";
-    char *psz_uri = make_URI( psz_file, NULL );
+    char *psz_uri = vlc_path2uri( psz_file, NULL );
 
     if( psz_uri == NULL )
         return VLC_EGENERIC;
@@ -133,7 +133,7 @@ int playlist_MLLoad( playlist_t *p_playlist )
     if( psz_file == NULL )
         return VLC_ENOMEM;
 
-    /* loosy check for media library file */
+    /* lousy check for media library file */
     struct stat st;
     if( vlc_stat( psz_file, &st ) )
     {
@@ -141,7 +141,7 @@ int playlist_MLLoad( playlist_t *p_playlist )
         return VLC_EGENERIC;
     }
 
-    char *psz_uri = make_URI( psz_file, "file/xspf-open" );
+    char *psz_uri = vlc_path2uri( psz_file, "file/xspf-open" );
     free( psz_file );
     if( psz_uri == NULL )
         return VLC_ENOMEM;
@@ -167,9 +167,7 @@ int playlist_MLLoad( playlist_t *p_playlist )
     pl_priv(p_playlist)->b_doing_ml = true;
     PL_UNLOCK;
 
-    stats_TimerStart( p_playlist, "ML Load", STATS_TIMER_ML_LOAD );
     input_Read( p_playlist, p_input );
-    stats_TimerStop( p_playlist,STATS_TIMER_ML_LOAD );
 
     PL_LOCK;
     pl_priv(p_playlist)->b_doing_ml = false;
@@ -203,10 +201,8 @@ int playlist_MLDump( playlist_t *p_playlist )
 
     strcat( psz_dirname, DIR_SEP "ml.xspf" );
 
-    stats_TimerStart( p_playlist, "ML Dump", STATS_TIMER_ML_DUMP );
     playlist_Export( p_playlist, psz_dirname, p_playlist->p_media_library,
                      "export-xspf" );
-    stats_TimerStop( p_playlist, STATS_TIMER_ML_DUMP );
 
     return VLC_SUCCESS;
 }
