@@ -36,95 +36,17 @@
 #include <assert.h>
 #include <limits.h>
 #include <errno.h>
-#ifdef UNDER_CE
-# include <mmsystem.h>
-#endif
-
-static vlc_threadvar_t thread_key;
-
-/**
- * Per-thread data
- */
-struct vlc_thread
-{
-    HANDLE         id;
-#ifdef UNDER_CE
-    HANDLE         cancel_event;
-#endif
-
-    bool           detached;
-    bool           killable;
-    bool           killed;
-    vlc_cleanup_t *cleaners;
-
-    void        *(*entry) (void *);
-    void          *data;
-};
 
+/*** Static mutex and condition variable ***/
 static vlc_mutex_t super_mutex;
 static vlc_cond_t  super_variable;
-extern vlc_rwlock_t config_lock, msg_lock;
 
-BOOL WINAPI DllMain (HINSTANCE, DWORD, LPVOID);
-
-BOOL WINAPI DllMain (HINSTANCE hinstDll, DWORD fdwReason, LPVOID lpvReserved)
-{
-    (void) hinstDll;
-    (void) lpvReserved;
-
-    switch (fdwReason)
-    {
-        case DLL_PROCESS_ATTACH:
-            vlc_mutex_init (&super_mutex);
-            vlc_cond_init (&super_variable);
-            vlc_threadvar_create (&thread_key, NULL);
-            vlc_rwlock_init (&config_lock);
-            vlc_rwlock_init (&msg_lock);
-            vlc_CPU_init ();
-            break;
-
-        case DLL_PROCESS_DETACH:
-            vlc_rwlock_destroy (&msg_lock);
-            vlc_rwlock_destroy (&config_lock);
-            vlc_threadvar_delete (&thread_key);
-            vlc_cond_destroy (&super_variable);
-            vlc_mutex_destroy (&super_mutex);
-            break;
-    }
-    return TRUE;
-}
-
-static void CALLBACK vlc_cancel_self (ULONG_PTR);
 
+/*** Common helpers ***/
 static DWORD vlc_WaitForMultipleObjects (DWORD count, const HANDLE *handles,
                                          DWORD delay)
 {
     DWORD ret;
-#ifdef UNDER_CE
-    HANDLE buf[count + 1];
-
-    struct vlc_thread *th = vlc_threadvar_get (thread_key);
-    if (th != NULL)
-    {
-        memcpy (buf, handles, count * sizeof(HANDLE));
-        buf[count++] = th->cancel_event;
-        handles = buf;
-    }
-
-    if (count == 0)
-    {
-         Sleep (delay);
-         ret = WAIT_TIMEOUT;
-    }
-    else
-        ret = WaitForMultipleObjects (count, handles, FALSE, delay);
-
-    if ((th != NULL) && (ret == WAIT_OBJECT_0 + count - 1))
-    {
-        vlc_cancel_self ((uintptr_t)th);
-        ret = WAIT_IO_COMPLETION;
-    }
-#else
     if (count == 0)
     {
         ret = SleepEx (delay, TRUE);
@@ -133,7 +55,7 @@ static DWORD vlc_WaitForMultipleObjects (DWORD count, const HANDLE *handles,
     }
     else
         ret = WaitForMultipleObjectsEx (count, handles, FALSE, delay, TRUE);
-#endif
+
     /* We do not abandon objects... this would be a bug */
     assert (ret < WAIT_ABANDONED_0 || WAIT_ABANDONED_0 + count - 1 < ret);
 
@@ -240,8 +162,9 @@ void vlc_mutex_unlock (vlc_mutex_t *p_mutex)
 /*** Condition variables ***/
 enum
 {
-    CLOCK_REALTIME=0, /* must be zero for VLC_STATIC_COND */
-    CLOCK_MONOTONIC,
+    VLC_CLOCK_STATIC=0, /* must be zero for VLC_STATIC_COND */
+    VLC_CLOCK_MONOTONIC,
+    VLC_CLOCK_REALTIME,
 };
 
 static void vlc_cond_init_common (vlc_cond_t *p_condvar, unsigned clock)
@@ -255,12 +178,12 @@ static void vlc_cond_init_common (vlc_cond_t *p_condvar, unsigned clock)
 
 void vlc_cond_init (vlc_cond_t *p_condvar)
 {
-    vlc_cond_init_common (p_condvar, CLOCK_MONOTONIC);
+    vlc_cond_init_common (p_condvar, VLC_CLOCK_MONOTONIC);
 }
 
 void vlc_cond_init_daytime (vlc_cond_t *p_condvar)
 {
-    vlc_cond_init_common (p_condvar, CLOCK_REALTIME);
+    vlc_cond_init_common (p_condvar, VLC_CLOCK_REALTIME);
 }
 
 void vlc_cond_destroy (vlc_cond_t *p_condvar)
@@ -270,7 +193,7 @@ void vlc_cond_destroy (vlc_cond_t *p_condvar)
 
 void vlc_cond_signal (vlc_cond_t *p_condvar)
 {
-    if (!p_condvar->handle)
+    if (!p_condvar->clock)
         return;
 
     /* This is suboptimal but works. */
@@ -279,7 +202,7 @@ void vlc_cond_signal (vlc_cond_t *p_condvar)
 
 void vlc_cond_broadcast (vlc_cond_t *p_condvar)
 {
-    if (!p_condvar->handle)
+    if (!p_condvar->clock)
         return;
 
     /* Wake all threads up (as the event HANDLE has manual reset) */
@@ -290,7 +213,7 @@ void vlc_cond_wait (vlc_cond_t *p_condvar, vlc_mutex_t *p_mutex)
 {
     DWORD result;
 
-    if (!p_condvar->handle)
+    if (!p_condvar->clock)
     {   /* FIXME FIXME FIXME */
         msleep (50000);
         return;
@@ -313,12 +236,6 @@ int vlc_cond_timedwait (vlc_cond_t *p_condvar, vlc_mutex_t *p_mutex,
 {
     DWORD result;
 
-    if (!p_condvar->handle)
-    {   /* FIXME FIXME FIXME */
-        msleep (50000);
-        return 0;
-    }
-
     do
     {
         vlc_testcancel ();
@@ -326,13 +243,17 @@ int vlc_cond_timedwait (vlc_cond_t *p_condvar, vlc_mutex_t *p_mutex,
         mtime_t total;
         switch (p_condvar->clock)
         {
-            case CLOCK_REALTIME: /* FIXME? sub-second precision */
+            case VLC_CLOCK_MONOTONIC:
+                total = mdate();
+                break;
+            case VLC_CLOCK_REALTIME: /* FIXME? sub-second precision */
                 total = CLOCK_FREQ * time (NULL);
                 break;
             default:
-                assert (p_condvar->clock == CLOCK_MONOTONIC);
-                total = mdate();
-                break;
+                assert (!p_condvar->clock);
+                /* FIXME FIXME FIXME */
+                msleep (50000);
+                return 0;
         }
         total = (deadline - total) / 1000;
         if( total < 0 )
@@ -381,93 +302,6 @@ void vlc_sem_wait (vlc_sem_t *sem)
     while (result == WAIT_IO_COMPLETION);
 }
 
-/*** Read/write locks */
-/* SRW (Slim Read Write) locks are available in Vista+ only */
-void vlc_rwlock_init (vlc_rwlock_t *lock)
-{
-    vlc_mutex_init (&lock->mutex);
-    vlc_cond_init (&lock->wait);
-    lock->readers = 0; /* active readers */
-    lock->writers = 0; /* waiting writers */
-    lock->writer = 0; /* ID of active writer */
-}
-
-void vlc_rwlock_destroy (vlc_rwlock_t *lock)
-{
-    vlc_cond_destroy (&lock->wait);
-    vlc_mutex_destroy (&lock->mutex);
-}
-
-void vlc_rwlock_rdlock (vlc_rwlock_t *lock)
-{
-    vlc_mutex_lock (&lock->mutex);
-    /* Recursive read-locking is allowed. With the infos available:
-     *  - the loosest possible condition (no active writer) is:
-     *     (lock->writer != 0)
-     *  - the strictest possible condition is:
-     *     (lock->writer != 0 || (lock->readers == 0 && lock->writers > 0))
-     *  or (lock->readers == 0 && (lock->writer != 0 || lock->writers > 0))
-     */
-    while (lock->writer != 0)
-    {
-        assert (lock->readers == 0);
-        vlc_cond_wait (&lock->wait, &lock->mutex);
-    }
-    if (unlikely(lock->readers == ULONG_MAX))
-        abort ();
-    lock->readers++;
-    vlc_mutex_unlock (&lock->mutex);
-}
-
-static void vlc_rwlock_rdunlock (vlc_rwlock_t *lock)
-{
-    vlc_mutex_lock (&lock->mutex);
-    assert (lock->readers > 0);
-
-    /* If there are no readers left, wake up a writer. */
-    if (--lock->readers == 0 && lock->writers > 0)
-        vlc_cond_signal (&lock->wait);
-    vlc_mutex_unlock (&lock->mutex);
-}
-
-void vlc_rwlock_wrlock (vlc_rwlock_t *lock)
-{
-    vlc_mutex_lock (&lock->mutex);
-    if (unlikely(lock->writers == ULONG_MAX))
-        abort ();
-    lock->writers++;
-    /* Wait until nobody owns the lock in either way. */
-    while ((lock->readers > 0) || (lock->writer != 0))
-        vlc_cond_wait (&lock->wait, &lock->mutex);
-    lock->writers--;
-    assert (lock->writer == 0);
-    lock->writer = GetCurrentThreadId ();
-    vlc_mutex_unlock (&lock->mutex);
-}
-
-static void vlc_rwlock_wrunlock (vlc_rwlock_t *lock)
-{
-    vlc_mutex_lock (&lock->mutex);
-    assert (lock->writer == GetCurrentThreadId ());
-    assert (lock->readers == 0);
-    lock->writer = 0; /* Write unlock */
-
-    /* Let reader and writer compete. Scheduler decides who wins. */
-    vlc_cond_broadcast (&lock->wait);
-    vlc_mutex_unlock (&lock->mutex);
-}
-
-void vlc_rwlock_unlock (vlc_rwlock_t *lock)
-{
-    /* Note: If the lock is held for reading, lock->writer is nul.
-     * If the lock is held for writing, only this thread can store a value to
-     * lock->writer. Either way, lock->writer is safe to fetch here. */
-    if (lock->writer != 0)
-        vlc_rwlock_wrunlock (lock);
-    else
-        vlc_rwlock_rdunlock (lock);
-}
-
 /*** Thread-specific variables (TLS) ***/
 struct vlc_threadvar
 {
@@ -524,19 +358,39 @@ void vlc_threadvar_delete (vlc_threadvar_t *p_tls)
 
 int vlc_threadvar_set (vlc_threadvar_t key, void *value)
 {
-    return TlsSetValue (key->id, value) ? ENOMEM : 0;
+    int saved = GetLastError ();
+    int val = TlsSetValue (key->id, value) ? ENOMEM : 0;
+
+    if (val == 0)
+        SetLastError(saved);
+    return val;
 }
 
 void *vlc_threadvar_get (vlc_threadvar_t key)
 {
-    return TlsGetValue (key->id);
+    int saved = GetLastError ();
+    void *value = TlsGetValue (key->id);
+
+    SetLastError(saved);
+    return value;
 }
 
 /*** Threads ***/
-void vlc_threads_setup (libvlc_int_t *p_libvlc)
+static vlc_threadvar_t thread_key;
+
+/** Per-thread data */
+struct vlc_thread
 {
-    (void) p_libvlc;
-}
+    HANDLE         id;
+
+    bool           detached;
+    bool           killable;
+    bool           killed;
+    vlc_cleanup_t *cleaners;
+
+    void        *(*entry) (void *);
+    void          *data;
+};
 
 static void vlc_thread_cleanup (struct vlc_thread *th)
 {
@@ -561,9 +415,6 @@ retry:
     if (th->detached)
     {
         CloseHandle (th->id);
-#ifdef UNDER_CE
-        CloseHandle (th->cancel_event);
-#endif
         free (th);
     }
 }
@@ -593,7 +444,6 @@ static int vlc_clone_attr (vlc_thread_t *p_handle, bool detached,
     th->cleaners = NULL;
 
     HANDLE hThread;
-#ifndef UNDER_CE
     /* When using the MSVCRT C library you have to use the _beginthreadex
      * function instead of CreateThread, otherwise you'll end up with
      * memory leaks and the signal functions not working (see Microsoft
@@ -609,26 +459,6 @@ static int vlc_clone_attr (vlc_thread_t *p_handle, bool detached,
     }
     hThread = (HANDLE)h;
 
-#else
-    th->cancel_event = CreateEvent (NULL, FALSE, FALSE, NULL);
-    if (th->cancel_event == NULL)
-    {
-        free (th);
-        return ENOMEM;
-    }
-
-    /* Not sure if CREATE_SUSPENDED + ResumeThread() is any useful on WinCE.
-     * Thread handles act up, too. */
-    hThread = CreateThread (NULL, 128*1024, vlc_entry, th,
-                            CREATE_SUSPENDED, NULL);
-    if (hThread == NULL)
-    {
-        CloseHandle (th->cancel_event);
-        free (th);
-        return ENOMEM;
-    }
-
-#endif
     /* Thread is suspended, so we can safely set th->id */
     th->id = hThread;
     if (p_handle != NULL)
@@ -652,15 +482,11 @@ void vlc_join (vlc_thread_t th, void **result)
 {
     do
         vlc_testcancel ();
-    while (vlc_WaitForSingleObject (th->id, INFINITE)
-                                                        == WAIT_IO_COMPLETION);
+    while (vlc_WaitForSingleObject (th->id, INFINITE) == WAIT_IO_COMPLETION);
 
     if (result != NULL)
         *result = th->data;
     CloseHandle (th->id);
-#ifdef UNDER_CE
-    CloseHandle (th->cancel_event);
-#endif
     free (th);
 }
 
@@ -694,11 +520,7 @@ static void CALLBACK vlc_cancel_self (ULONG_PTR self)
 
 void vlc_cancel (vlc_thread_t th)
 {
-#ifndef UNDER_CE
     QueueUserAPC (vlc_cancel_self, th->id, (uintptr_t)th);
-#else
-    SetEvent (th->cancel_event);
-#endif
 }
 
 int vlc_savecancel (void)
@@ -737,11 +559,7 @@ void vlc_testcancel (void)
 
         th->data = NULL; /* TODO: special value? */
         vlc_thread_cleanup (th);
-#ifndef UNDER_CE
         _endthreadex(0);
-#else
-        ExitThread(0);
-#endif
     }
 }
 
@@ -777,21 +595,112 @@ void vlc_control_cancel (int cmd, ...)
     va_end (ap);
 }
 
-
 /*** Clock ***/
+static CRITICAL_SECTION clock_lock;
+
+static mtime_t mdate_giveup (void)
+{
+    abort ();
+}
+
+static mtime_t (*mdate_selected) (void) = mdate_giveup;
+
 mtime_t mdate (void)
+{
+    return mdate_selected ();
+}
+
+static union
+{
+    struct
+    {
+#if (_WIN32_WINNT < 0x0601)
+        BOOL (*query) (PULONGLONG);
+#endif
+    } interrupt;
+    struct
+    {
+#if (_WIN32_WINNT < 0x0600)
+        ULONGLONG (*get) (void);
+#endif
+    } tick;
+    struct
+    {
+        LARGE_INTEGER freq;
+    } perf;
+} clk;
+
+static mtime_t mdate_interrupt (void)
+{
+    ULONGLONG ts;
+    BOOL ret;
+
+#if (_WIN32_WINNT >= 0x0601)
+    ret = QueryUnbiasedInterruptTime (&ts);
+#else
+    ret = clk.interrupt.query (&ts);
+#endif
+    if (unlikely(!ret))
+        abort ();
+
+    /* hundreds of nanoseconds */
+    static_assert ((10000000 % CLOCK_FREQ) == 0, "Broken frequencies ratio");
+    return ts / (10000000 / CLOCK_FREQ);
+}
+
+static mtime_t mdate_tick (void)
+{
+#if (_WIN32_WINNT >= 0x0600)
+    ULONGLONG ts = GetTickCount64 ();
+#else
+    ULONGLONG ts = clk.tick.get ();
+#endif
+
+    /* milliseconds */
+    static_assert ((CLOCK_FREQ % 1000) == 0, "Broken frequencies ratio");
+    return ts * (CLOCK_FREQ / 1000);
+}
+#if !VLC_WINSTORE_APP
+#include <mmsystem.h>
+static mtime_t mdate_multimedia (void)
+{
+     DWORD ts = timeGetTime ();
+
+    /* milliseconds */
+    static_assert ((CLOCK_FREQ % 1000) == 0, "Broken frequencies ratio");
+    return ts * (CLOCK_FREQ / 1000);
+}
+#endif
+
+static mtime_t mdate_perf (void)
 {
     /* We don't need the real date, just the value of a high precision timer */
-    LARGE_INTEGER counter, freq;
-    if (!QueryPerformanceCounter (&counter)
-     || !QueryPerformanceFrequency (&freq))
-        abort();
+    LARGE_INTEGER counter;
+    if (!QueryPerformanceCounter (&counter))
+        abort ();
 
     /* Convert to from (1/freq) to microsecond resolution */
     /* We need to split the division to avoid 63-bits overflow */
-    lldiv_t d = lldiv (counter.QuadPart, freq.QuadPart);
+    lldiv_t d = lldiv (counter.QuadPart, clk.perf.freq.QuadPart);
 
-    return (d.quot * 1000000) + ((d.rem * 1000000) / freq.QuadPart);
+    return (d.quot * 1000000) + ((d.rem * 1000000) / clk.perf.freq.QuadPart);
+}
+
+static mtime_t mdate_wall (void)
+{
+    FILETIME ts;
+    ULARGE_INTEGER s;
+
+#if (_WIN32_WINNT >= 0x0602) && !VLC_WINSTORE_APP
+    GetSystemTimePreciseAsFileTime (&ts);
+#else
+    GetSystemTimeAsFileTime (&ts);
+#endif
+    s.LowPart = ts.dwLowDateTime;
+    s.HighPart = ts.dwHighDateTime;
+    /* hundreds of nanoseconds */
+    static_assert ((10000000 % CLOCK_FREQ) == 0, "Broken frequencies ratio");
+    return s.QuadPart / (10000000 / CLOCK_FREQ);
 }
 
 #undef mwait
@@ -816,21 +725,150 @@ void msleep (mtime_t delay)
     mwait (mdate () + delay);
 }
 
+static void SelectClockSource (vlc_object_t *obj)
+{
+    EnterCriticalSection (&clock_lock);
+    if (mdate_selected != mdate_giveup)
+    {
+        LeaveCriticalSection (&clock_lock);
+        return;
+    }
+
+#if VLC_WINSTORE_APP
+    const char *name = "perf";
+#else
+    const char *name = "multimedia";
+#endif
+    char *str = var_InheritString (obj, "clock-source");
+    if (str != NULL)
+        name = str;
+    if (!strcmp (name, "interrupt"))
+    {
+        msg_Dbg (obj, "using interrupt time as clock source");
+#if (_WIN32_WINNT < 0x0601)
+        HANDLE h = GetModuleHandle (_T("kernel32.dll"));
+        if (unlikely(h == NULL))
+            abort ();
+        clk.interrupt.query = (void *)GetProcAddress (h,
+                                                      "QueryUnbiasedInterruptTime");
+        if (unlikely(clk.interrupt.query == NULL))
+            abort ();
+#endif
+        mdate_selected = mdate_interrupt;
+    }
+    else
+    if (!strcmp (name, "tick"))
+    {
+        msg_Dbg (obj, "using Windows time as clock source");
+#if (_WIN32_WINNT < 0x0600)
+        HANDLE h = GetModuleHandle (_T("kernel32.dll"));
+        if (unlikely(h == NULL))
+            abort ();
+        clk.tick.get = (void *)GetProcAddress (h, "GetTickCount64");
+        if (unlikely(clk.tick.get == NULL))
+            abort ();
+#endif
+        mdate_selected = mdate_tick;
+    }
+#if !VLC_WINSTORE_APP
+    else
+    if (!strcmp (name, "multimedia"))
+    {
+        TIMECAPS caps;
+
+        msg_Dbg (obj, "using multimedia timers as clock source");
+        if (timeGetDevCaps (&caps, sizeof (caps)) != MMSYSERR_NOERROR)
+            abort ();
+        msg_Dbg (obj, " min period: %u ms, max period: %u ms",
+                 caps.wPeriodMin, caps.wPeriodMax);
+        mdate_selected = mdate_multimedia;
+    }
+#endif
+    else
+    if (!strcmp (name, "perf"))
+    {
+        msg_Dbg (obj, "using performance counters as clock source");
+        if (!QueryPerformanceFrequency (&clk.perf.freq))
+            abort ();
+        msg_Dbg (obj, " frequency: %llu Hz", clk.perf.freq.QuadPart);
+        mdate_selected = mdate_perf;
+    }
+    else
+    if (!strcmp (name, "wall"))
+    {
+        msg_Dbg (obj, "using system time as clock source");
+        mdate_selected = mdate_wall;
+    }
+    else
+    {
+        msg_Err (obj, "invalid clock source \"%s\"", name);
+        abort ();
+    }
+    LeaveCriticalSection (&clock_lock);
+    free (str);
+}
+
+#define xstrdup(str) (strdup(str) ?: (abort(), NULL))
+
+size_t EnumClockSource (vlc_object_t *obj, const char *var,
+                        char ***vp, char ***np)
+{
+    const size_t max = 6;
+    char **values = xmalloc (sizeof (*values) * max);
+    char **names = xmalloc (sizeof (*names) * max);
+    size_t n = 0;
+
+#if (_WIN32_WINNT < 0x0601)
+    DWORD version = LOWORD(GetVersion());
+    version = (LOBYTE(version) << 8) | (HIBYTE(version) << 0);
+#endif
+
+    values[n] = xstrdup ("");
+    names[n] = xstrdup (_("Auto"));
+    n++;
+#if (_WIN32_WINNT < 0x0601)
+    if (version >= 0x0601)
+#endif
+    {
+        values[n] = xstrdup ("interrupt");
+        names[n] = xstrdup ("Interrupt time");
+        n++;
+    }
+#if (_WIN32_WINNT < 0x0600)
+    if (version >= 0x0600)
+#endif
+    {
+        values[n] = xstrdup ("tick");
+        names[n] = xstrdup ("Windows time");
+        n++;
+    }
+#if !VLC_WINSTORE_APP
+    values[n] = xstrdup ("multimedia");
+    names[n] = xstrdup ("Multimedia timers");
+    n++;
+#endif
+    values[n] = xstrdup ("perf");
+    names[n] = xstrdup ("Performance counters");
+    n++;
+    values[n] = xstrdup ("wall");
+    names[n] = xstrdup ("System time (DANGEROUS!)");
+    n++;
+
+    *vp = values;
+    *np = names;
+    (void) obj; (void) var;
+    return n;
+}
+
 
 /*** Timers ***/
 struct vlc_timer
 {
-#ifndef UNDER_CE
     HANDLE handle;
-#else
-    unsigned id;
-    unsigned interval;
-#endif
     void (*func) (void *);
     void *data;
 };
 
-#ifndef UNDER_CE
 static void CALLBACK vlc_timer_do (void *val, BOOLEAN timeout)
 {
     struct vlc_timer *timer = val;
@@ -838,26 +876,6 @@ static void CALLBACK vlc_timer_do (void *val, BOOLEAN timeout)
     assert (timeout);
     timer->func (timer->data);
 }
-#else
-static void CALLBACK vlc_timer_do (unsigned timer_id, unsigned msg,
-                                   DWORD_PTR user, DWORD_PTR unused1,
-                                   DWORD_PTR unused2)
-{
-    struct vlc_timer *timer = (struct vlc_timer *) user;
-    assert (timer_id == timer->id);
-    (void) msg;
-    (void) unused1;
-    (void) unused2;
-
-    timer->func (timer->data);
-
-    if (timer->interval)
-    {
-        mtime_t interval = timer->interval * 1000;
-        vlc_timer_schedule (timer, false, interval, interval);
-    }
-}
-#endif
 
 int vlc_timer_create (vlc_timer_t *id, void (*func) (void *), void *data)
 {
@@ -867,46 +885,26 @@ int vlc_timer_create (vlc_timer_t *id, void (*func) (void *), void *data)
         return ENOMEM;
     timer->func = func;
     timer->data = data;
-#ifndef UNDER_CE
     timer->handle = INVALID_HANDLE_VALUE;
-#else
-    timer->id = 0;
-    timer->interval = 0;
-#endif
     *id = timer;
     return 0;
 }
 
 void vlc_timer_destroy (vlc_timer_t timer)
 {
-#ifndef UNDER_CE
     if (timer->handle != INVALID_HANDLE_VALUE)
         DeleteTimerQueueTimer (NULL, timer->handle, INVALID_HANDLE_VALUE);
-#else
-    if (timer->id)
-        timeKillEvent (timer->id);
-    /* FIXME: timers that have not yet completed will trigger use-after-free */
-#endif
     free (timer);
 }
 
 void vlc_timer_schedule (vlc_timer_t timer, bool absolute,
                          mtime_t value, mtime_t interval)
 {
-#ifndef UNDER_CE
     if (timer->handle != INVALID_HANDLE_VALUE)
     {
         DeleteTimerQueueTimer (NULL, timer->handle, NULL);
         timer->handle = INVALID_HANDLE_VALUE;
     }
-#else
-    if (timer->id)
-    {
-        timeKillEvent (timer->id);
-        timer->id = 0;
-        timer->interval = 0;
-    }
-#endif
     if (value == 0)
         return; /* Disarm */
 
@@ -915,28 +913,8 @@ void vlc_timer_schedule (vlc_timer_t timer, bool absolute,
     value = (value + 999) / 1000;
     interval = (interval + 999) / 1000;
 
-#ifndef UNDER_CE
     if (!CreateTimerQueueTimer (&timer->handle, NULL, vlc_timer_do, timer,
                                 value, interval, WT_EXECUTEDEFAULT))
-#else
-    TIMECAPS caps;
-    timeGetDevCaps (&caps, sizeof(caps));
-
-    unsigned delay = value;
-    delay = __MAX(delay, caps.wPeriodMin);
-    delay = __MIN(delay, caps.wPeriodMax);
-
-    unsigned event = TIME_ONESHOT;
-
-    if (interval == delay)
-        event = TIME_PERIODIC;
-    else if (interval)
-        timer->interval = interval;
-
-    timer->id = timeSetEvent (delay, delay / 20, vlc_timer_do, (DWORD) timer,
-                              event);
-    if (!timer->id)
-#endif
         abort ();
 }
 
@@ -950,12 +928,46 @@ unsigned vlc_timer_getoverrun (vlc_timer_t timer)
 /*** CPU ***/
 unsigned vlc_GetCPUCount (void)
 {
-#ifndef UNDER_CE
-    DWORD_PTR process;
-    DWORD_PTR system;
+    SYSTEM_INFO systemInfo;
 
-    if (GetProcessAffinityMask (GetCurrentProcess(), &process, &system))
-        return popcount (system);
-#endif
-     return 1;
+    GetNativeSystemInfo(&systemInfo);
+
+    return systemInfo.dwNumberOfProcessors;
+}
+
+
+/*** Initialization ***/
+void vlc_threads_setup (libvlc_int_t *p_libvlc)
+{
+    SelectClockSource (VLC_OBJECT(p_libvlc));
+}
+
+extern vlc_rwlock_t config_lock;
+BOOL WINAPI DllMain (HINSTANCE, DWORD, LPVOID);
+
+BOOL WINAPI DllMain (HINSTANCE hinstDll, DWORD fdwReason, LPVOID lpvReserved)
+{
+    (void) hinstDll;
+    (void) lpvReserved;
+
+    switch (fdwReason)
+    {
+        case DLL_PROCESS_ATTACH:
+            InitializeCriticalSection (&clock_lock);
+            vlc_mutex_init (&super_mutex);
+            vlc_cond_init (&super_variable);
+            vlc_threadvar_create (&thread_key, NULL);
+            vlc_rwlock_init (&config_lock);
+            vlc_CPU_init ();
+            break;
+
+        case DLL_PROCESS_DETACH:
+            vlc_rwlock_destroy (&config_lock);
+            vlc_threadvar_delete (&thread_key);
+            vlc_cond_destroy (&super_variable);
+            vlc_mutex_destroy (&super_mutex);
+            DeleteCriticalSection (&clock_lock);
+            break;
+    }
+    return TRUE;
 }
